"""
tooledit
===============================================================================

Implementation of the edit tool for the glyph editor
"""
import logging
from math import atan2, degrees

import wx
from wbBase.tools import get_wxBrush, get_wxPen
from wbDefcon import Anchor, Component, Guideline, Point
from wbDefcon.pens import ContourHit

from ....dialog.anchorDialog import AnchorDialog
from ....dialog.componentDialog import ComponentDialog
from ....dialog.guidelineDialog import GuidelineDialog
from ..cursor import cursors
from .base import IDLE, RUBBERBANDING, WORKING, GlyphTool

log = logging.getLogger(__name__)


class ToolEdit(GlyphTool):
    """
    Tool to edit contours, anchors, components, guidelines
    """

    brush = get_wxBrush(wx.Colour(200, 200, 200, 64))
    pen = get_wxPen(wx.Colour(128, 128, 128, 128))

    def __init__(self, parent):
        super().__init__(parent)
        self.Bind(wx.EVT_LEFT_DCLICK, self.on_LEFT_DCLICK)
        # Keyboard Events
        self.Bind(wx.EVT_KEY_DOWN, self.on_KEY_DOWN)

    def setCursor(self):
        if self.subject is None:
            self.canvas.SetCursor(cursors["Default"])
        elif isinstance(self.subject, Point):
            if self.subject.selected:
                self.canvas.SetCursor(cursors["EditSelection"])
            else:
                self.canvas.SetCursor(cursors["EditPoint"])
        elif isinstance(self.subject, Anchor):
            if self.subject.selected:
                self.canvas.SetCursor(cursors["EditSelection"])
            else:
                self.canvas.SetCursor(cursors["EditAnchor"])
        elif isinstance(self.subject, Component):
            if self.subject.selected:
                self.canvas.SetCursor(cursors["EditSelection"])
            else:
                self.canvas.SetCursor(cursors["EditComponent"])
        elif isinstance(self.subject, Guideline):
            if self.subject.selected:
                self.canvas.SetCursor(cursors["EditSelection"])
            else:
                self.canvas.SetCursor(cursors["EditGuideline"])
        elif isinstance(self.subject, ContourHit):
            if self.subject.selected:
                self.canvas.SetCursor(cursors["EditSelection"])
            else:
                self.canvas.SetCursor(cursors["EditContour"])

    def moveSelection(self, dx, dy):
        glyph = self.glyph
        for contour in glyph:
            for point in contour:
                if point.selected:
                    point.move((dx, dy))
                    self.contoursChanged = True
        for anchor in glyph.anchors:
            if anchor.selected:
                anchor.move((dx, dy))
                self.anchorsChanged = True
        for component in glyph.components:
            if component.selected:
                component.move((dx, dy))
                self.componentsChanged = True
        for guideline in glyph.guidelines:
            if guideline.selected:
                guideline.move((dx, dy))
                self.glyphGuidelinesChanged = True
        for guideline in glyph.font.guidelines:
            if guideline.selected:
                guideline.move((dx, dy))
                self.fontGuidelinesChanged = True

    # -------------------------------------------------------------------------
    # Event handler methods
    # -------------------------------------------------------------------------

    def on_LEFT_DCLICK(self, event):
        if isinstance(self.subject, Component):
            # show ComponentDialog to edit component details
            component = self.subject
            with ComponentDialog(self.canvas, component) as componentDialog:
                if componentDialog.ShowModal() == wx.ID_OK:
                    component.baseGlyph = componentDialog.baseGlyph
                    component.offset = componentDialog.offset
                    component.scale = componentDialog.scale
        elif isinstance(self.subject, Anchor):
            # show AnchorDialog to edit anchor details
            anchor = self.subject
            with AnchorDialog(self.canvas, anchor.asDict()) as anchorDialog:
                if anchorDialog.ShowModal() == wx.ID_OK:
                    d = anchorDialog.anchorDict
                    anchor.x = d["x"]
                    anchor.y = d["y"]
                    anchor.name = d["name"]
                    anchor.color = d["color"]
        elif isinstance(self.subject, Guideline):
            # show GuidelineDialog to edit guideline details
            guideline = self.subject
            with GuidelineDialog(
                self.canvas, guideline.getDataForSerialization()
            ) as guidelineDiaolog:
                if guidelineDiaolog.ShowModal() == wx.ID_OK:
                    guideline.setDataFromSerialization(guidelineDiaolog.guidelineDict)
        elif isinstance(self.subject, ContourHit):
            # select whole contour
            self.subject.contour.selected = not self.subject.contour.selected
            self.canvas.Refresh()

    def on_LEFT_DOWN(self, event):
        super().on_LEFT_DOWN(event)
        glyph = self.glyph
        if self.subject is None:
            self.state = RUBBERBANDING
        elif glyph is not None:
            glyph.undoManager.saveState()
            glyph.disableNotifications()
            self.state = WORKING
        event.Skip()

    def on_LEFT_UP(self, event):
        log.debug("on_LEFT_UP: %r", self.state)
        glyph = self.glyph
        if glyph is not None:
            if self.state == RUBBERBANDING:
                x0 = self.canvas.screenToCanvasX(self.xClick)
                y0 = self.canvas.screenToCanvasY(self.yClick)
                x1 = self.canvas.screenToCanvasX(self.x)
                y1 = self.canvas.screenToCanvasY(self.y)
                stack = self.activeStack
                glyph.selectRect(
                    (x0, y0, x1, y1),
                    addToSelection=event.ShiftDown(),
                    selectPoints=not stack["OnCurvePoints"].locked,
                    selectComponents=not stack["GlyphComponent"].locked,
                    selectAnchors=not stack["Anchors"].locked,
                    selectGuidelines=not stack["GlyphGuide"].locked,
                )
                glyph.font.selectRect(
                    (x0, y0, x1, y1),
                    addToSelection=event.ShiftDown(),
                    selectGuidelines=not stack["GlyphGuide"].locked,
                )
            elif self.state == WORKING:
                try:
                    glyph.enableNotifications()
                except KeyError:
                    pass
                if not self.subject.selected:
                    if not event.ShiftDown():
                        self.unselectAll()
                    self.subject.selected = True
                    if (
                        isinstance(self.subject, Point)
                        and self.subject.segmentType is not None
                    ):
                        for contour in glyph:
                            if self.subject in contour.onCurvePoints:
                                contour.updateSelection()
                self.postNotifications()
        super().on_LEFT_UP(event)

    def on_RIGHT_DOWN(self, event):
        if self.state == WORKING and not self.subject.selected:
            # right click while left is down on unselected objets
            glyph = self.glyph
            if isinstance(self.subject, Anchor):
                glyph.removeAnchor(self.subject)
            elif isinstance(self.subject, Component):
                glyph.removeComponent(self.subject)
                self.componentsChanged = True
            elif isinstance(self.subject, Guideline):
                if self.subject.glyph is None:
                    glyph.font.removeGuideline(self.subject)
                else:
                    glyph.removeGuideline(self.subject)
            elif isinstance(self.subject, Point):
                point = self.subject
                for contour in glyph:
                    if point in contour:
                        segmentIndex = contour.segmentIndex(point)
                        if segmentIndex >= 0:
                            glyph.enableNotifications()
                            if point.segmentType is None:
                                # right click on off-curve point, convert to line
                                segment = contour.segments[segmentIndex]
                                segment[-1].segmentType = "line"
                                segment[-1].smooth = False
                                for p in segment[:-1]:
                                    contour.removePoint(p)
                            else:
                                # right click on on-curve point, remove the segment
                                contour.removeSegment(segmentIndex, True)
                                if len(contour) == 0:
                                    glyph.removeContour(contour)
                            self.contoursChanged = True
            elif isinstance(self.subject, ContourHit):
                # right click on contour, split the segment
                glyph.enableNotifications()
                if event.AltDown():
                    self.subject.convertToCurve()
                else:
                    self.subject.splitAndInsertPoint()
                self.contoursChanged = True
            if self.contoursChanged or self.componentsChanged:
                glyph.destroyRepresentation("outlineErrors")
            self.state = IDLE
            self.canvas.Refresh()
            self.postNotifications()
        else:
            super().on_RIGHT_DOWN(event)

    def on_MOTION(self, event):
        super().on_MOTION(event)
        glyph = self.glyph
        if glyph is not None:
            canvas_x = self.canvas.screenToCanvasX(self.x)
            canvas_y = self.canvas.screenToCanvasY(self.y)
            if event.Dragging():
                if self.state == RUBBERBANDING:
                    x = min(self.xClick, self.xPrev)
                    y = min(self.yClick, self.yPrev)
                    w = abs(self.xClick - self.xPrev)
                    h = abs(self.yClick - self.yPrev)
                    rubberband = wx.Rect(x, y, w, h)
                    x = min(self.xClick, self.x)
                    y = min(self.yClick, self.y)
                    w = abs(self.xClick - self.x)
                    h = abs(self.yClick - self.y)
                    rubberband.Union(wx.Rect(x, y, w, h))
                    rubberband.Inflate(1, 1)
                    self.canvas.RefreshRect(rubberband)
                elif self.state == WORKING:
                    dx = self.canvas.screenToCanvasXrel(self.x - self.xPrev)
                    dy = self.canvas.screenToCanvasYrel(self.y - self.yPrev)
                    if dx or dy:
                        if self.subject.selected:
                            self.moveSelection(dx, dy)
                        else:
                            self.unselectAll()
                            if isinstance(self.subject, Point):
                                if self.subject.segmentType is None:
                                    self.subject.move((dx, dy))
                                    self.subject.selected = True
                                else:
                                    self.subject.selected = True
                                    for contour in glyph:
                                        if self.subject in contour.onCurvePoints:
                                            contour.updateSelection()
                                            for point in contour:
                                                if point.selected:
                                                    point.move((dx, dy))
                                self.contoursChanged = True
                            elif isinstance(self.subject, Component):
                                self.subject.move((dx, dy))
                                self.componentsChanged = True
                            elif isinstance(self.subject, Anchor):
                                self.subject.move((dx, dy))
                                self.anchorsChanged = True
                            elif isinstance(self.subject, Guideline):
                                if event.AltDown():
                                    self.subject.angle = degrees(
                                        atan2(
                                            canvas_y - self.subject.y,
                                            canvas_x - self.subject.x,
                                        )
                                    )
                                else:
                                    self.subject.move((dx, dy))
                                if self.subject.glyph is None:
                                    self.fontGuidelinesChanged = True
                                else:
                                    self.glyphGuidelinesChanged = True
                        if any(
                            (
                                self.contoursChanged,
                                self.anchorsChanged,
                                self.componentsChanged,
                            )
                        ):
                            glyph.destroyRepresentation("outlineErrors")
                        self.canvas.Refresh()
            else:
                if self.state == IDLE:
                    self.hitTest()
                    self.setCursor()
        event.Skip()

    def on_KEY_DOWN(self, event):
        if self.isAnySelected():
            key = event.GetKeyCode()
            modifiers = event.GetModifiers()
            glyph = self.glyph
            glyph.undoManager.saveState()
            if key in (wx.WXK_LEFT, wx.WXK_RIGHT, wx.WXK_UP, wx.WXK_DOWN):
                # move selection with arrow keys
                distance = 1
                if modifiers == wx.MOD_SHIFT:
                    distance = 10
                elif modifiers == wx.MOD_CONTROL:
                    distance = 100
                elif modifiers == wx.MOD_SHIFT | wx.MOD_CONTROL:
                    distance = 1000
                if key == wx.WXK_LEFT:
                    move = (-distance, 0)
                elif key == wx.WXK_RIGHT:
                    move = (distance, 0)
                elif key == wx.WXK_UP:
                    move = (0, distance)
                elif key == wx.WXK_DOWN:
                    move = (0, -distance)
                self.moveSelection(*move)
            elif key == wx.WXK_DELETE:
                # delete selection
                for contour in reversed(glyph):
                    if contour.selected:
                        glyph.removeContour(contour)
                        self.contoursChanged = True
                    else:
                        for point in reversed(contour):
                            if point.selected:
                                contour.removePoint(point)
                                self.contoursChanged = True
                for anchor in glyph.anchors:
                    if anchor.selected:
                        glyph.removeAnchor(anchor)
                        self.anchorsChanged = True
                for component in glyph.components:
                    if component.selected:
                        glyph.removeComponent(component)
                        self.componentsChanged = True
                for guideline in glyph.guidelines:
                    if guideline.selected:
                        glyph.removeGuideline(guideline)
                        self.glyphGuidelinesChanged = True
                for guideline in glyph.font.guidelines:
                    if guideline.selected:
                        glyph.font.removeGuideline(guideline)
                        self.fontGuidelinesChanged = True
            if self.contoursChanged or self.componentsChanged:
                glyph.destroyRepresentation("outlineErrors")
            self.postNotifications()
            self.canvas.Refresh()
        else:
            event.Skip()

    def draw(self, gc):
        if self.state == RUBBERBANDING and None not in (
            self.xClick,
            self.yClick,
            self.x,
            self.y,
        ):
            gc.SetPen(self.pen)
            gc.SetBrush(self.brush)
            gc.DrawRectangle(
                min(self.xClick, self.x),
                min(self.yClick, self.y),
                abs(self.xClick - self.x),
                abs(self.yClick - self.y),
            )



"""
libControl
===============================================================================
"""
import logging
from pprint import PrettyPrinter
import wx
from wbDefcon import Lib, Font, Layer, Glyph

log = logging.getLogger(__name__)
pp = PrettyPrinter(indent=2).pformat


class LibTree(wx.TreeCtrl):
    def __init__(
        self,
        parent,
        id=wx.ID_ANY,
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=wx.SP_LIVE_UPDATE | wx.SP_NOBORDER,
    ):
        wx.TreeCtrl.__init__(self, parent, id, pos, size, style)
        self.root = None
        self.lib = None
        # Connect Events
        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.on_TREE_SEL_CHANGED)

    def updateLib(self, lib):
        if self.Count:
            self.DeleteAllItems()
        self.root = self.AddRoot("lib", data=lib)
        for libKey in sorted(lib.keys()):
            self.addLibData(self.root, libKey, lib[libKey])
        self.Expand(self.root)
        self.Refresh()

    def addLibData(self, parent, label, data):
        item = self.AppendItem(parent, label, data=data)
        if isinstance(data, (list, tuple)):
            for i, d in enumerate(data):
                self.addLibData(item, str(i), d)
        elif isinstance(data, dict):
            for libKey in sorted(data.keys()):
                self.addLibData(item, libKey, data[libKey])
        elif isinstance(data, (str, int, float, bool)):
            pass
        else:
            log.error("unhandled: %s, %r", label, data)
        return item

    # -------------------------------------------------------------------------
    # Event handler methods
    # -------------------------------------------------------------------------

    def on_TREE_SEL_CHANGED(self, event):
        itemID = event.GetItem()
        if itemID:
            item = self.GetItemData(itemID)
            value = f"Type:\n{type(item)}\n\n"
            value += f"Value:\n{pp(item)}\n"
            self.Parent.libValueCtrl.Value = value
        event.Skip()


class LibControl(wx.SplitterWindow):
    parentType = None

    def __init__(
        self,
        parent,
        id=wx.ID_ANY,
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=wx.SP_LIVE_UPDATE | wx.SP_NOBORDER,
    ):
        wx.SplitterWindow.__init__(self, parent, id, pos, size, style)
        self._lib = None
        self.SetSashGravity(0.5)
        self.SetMinimumPaneSize(100)
        self.libTreeCtrl = LibTree(
            parent=self,
            id=wx.ID_ANY,
            pos=wx.DefaultPosition,
            size=wx.DefaultSize,
            style=wx.TR_DEFAULT_STYLE
            | wx.TR_HAS_BUTTONS
            | wx.TR_SINGLE
            | wx.TR_TWIST_BUTTONS
            | wx.NO_BORDER,
        )
        self.libValueCtrl = wx.TextCtrl(
            parent=self,
            id=wx.ID_ANY,
            value=wx.EmptyString,
            pos=wx.DefaultPosition,
            size=wx.DefaultSize,
            style=wx.TE_MULTILINE | wx.TE_READONLY,
        )
        self.SplitVertically(self.libTreeCtrl, self.libValueCtrl, 100)
        self.Bind(wx.EVT_IDLE, self.on_IDLE)

    @property
    def lib(self):
        if not self._lib:
            self.lib = self.Validator.lib
        return self._lib

    @lib.setter
    def lib(self, lib):
        assert isinstance(lib, Lib)
        assert isinstance(lib.getParent(), self.parentType)
        del self.lib
        self._lib = lib
        self._lib.addObserver(self, "handleNotification", None)
        self.libTreeCtrl.updateLib(lib)

    @lib.deleter
    def lib(self):
        if isinstance(self._lib, Lib):
            self._lib.removeObserver(self, None)
        self._lib = None

    def on_IDLE(self, event):
        self.SetSashPosition(200)
        self.Unbind(wx.EVT_IDLE)

    def handleNotification(self, notification):
        log.debug("notification %r, %r, %r", self, notification.name, notification.data)
        if notification.name == self._lib.clearNotificationName:
            self.libTreeCtrl.DeleteAllItems()
        else:
            self.libTreeCtrl.updateLib(self.lib)
        self.libValueCtrl.Value = wx.EmptyString
        self.Refresh()

    def TransferDataToWindow(self):
        __ = self.lib
        return super().TransferDataToWindow()

class FontLibControl(LibControl):
    parentType = Font


class LayerLibControl(LibControl):
    parentType = Layer


class GlyphLibControl(LibControl):
    parentType = Glyph


class LibControlValidator(wx.Validator):
    def __init__(self, getLib):
        super().__init__()
        self._getLib = getLib
        self._lib = None

    @property
    def lib(self):
        if not self._lib:
            self._lib = self._getLib()
        return self._lib

    def Clone(self):
        return self.__class__(self._getLib)

    def Validate(self, win):
        return True

    def TransferToWindow(self):
        ctrl = self.GetWindow()
        ctrl.lib = self.lib
        return True

    def TransferFromWindow(self):
        return True



"""
base
===============================================================================

Base for all Glyph Commands
"""
from __future__ import annotations
import logging
from typing import List

from .parameter import GlyphCmdParameter, ParamEditableEnumeration

log = logging.getLogger(__name__)


class GlyphCommand:
    """Base class for Glyph-Commands"""

    name: str = "Command"
    parameters: List[GlyphCmdParameter] = []

    def __init__(self, **kwargs):
        self._setupParameters()
        for name, value in kwargs.items():
            assert self.has_parameter(name)
            assert isinstance(
                value, self.parameter(name).type
            ), "Expected instance of %r, got %r" % (self.parameter(name).type, value)
            setattr(self, name, value)

    def __str__(self) -> str:
        return "%s: %s" % (self.name, self.paramString)

    def __repr__(self) -> str:
        return "<%s: %s>" % (self.__class__.__name__, self.paramString)

    def __setattr__(self, name: str, value):
        parameter = self.get_parameter(name)
        if isinstance(parameter, ParamEditableEnumeration) and isinstance(value, int):
            value = parameter.labels[value]
        super().__setattr__(name, value)

    @property
    def paramString(self) -> str:
        params = []
        for param in self.parameters:
            params.append(
                "%s=%s" % (param.name, param.toString(getattr(self, param.name)))
            )
        return "; ".join(params)

    @classmethod
    def parameter(cls, name: str):
        for parameter in cls.parameters:
            if parameter.name == name:
                return parameter
        raise KeyError('Parameter "%s" not defined' % name)

    @classmethod
    def fromString(cls, string: str):
        if cls.parameters:
            params = dict(s.strip().split("=") for s in string.split(";"))
            for name in params:
                params[name] = cls.parameter(name).fromString(params[name])
        else:
            params = {}
        return cls(**params)

    @property
    def pgProperties(self):
        return [p.pgProperty(getattr(self, p.name)) for p in self.parameters]

    def _setupParameters(self) -> None:
        for parameter in self.parameters:
            assert isinstance(parameter, GlyphCmdParameter), (
                "expected <GlyphCmdParameter>, got %r" % parameter
            )
            setattr(self, parameter.name, parameter.default)

    def _checkParameters(self) -> bool:
        for parameter in self.parameters:
            if not hasattr(self, parameter.name):
                log.debug(
                    "Paramerter check failed: command %r has no parameter %r",
                    self.name,
                    parameter.name,
                )
                return False
            value = getattr(self, parameter.name)
            if value is None and not parameter.allowNone:
                log.debug(
                    "Paramerter check for command %r failed: %r is None, which is not allowed",
                    self.name,
                    parameter.name,
                )
                return False
            if isinstance(parameter, ParamEditableEnumeration):
                if isinstance(value, int):
                    return value in range(len(parameter.labels))
            if not isinstance(value, parameter.type):
                log.debug(
                    "Paramerter check for command %r failed: %r is of type %r, but %r is expected",
                    self.name,
                    parameter.name,
                    type(value),
                    parameter.type,
                )
                return False
        return True

    def has_parameter(self, name:str) -> bool:
        for parameter in self.parameters:
            if parameter.name == name:
                return True
        return False

    def get_parameter(self, name:str, default=None):
        for parameter in self.parameters:
            if parameter.name == name:
                return parameter
        return default

    def _execute(self, glyph):
        """override this in sub classes"""
        raise NotImplementedError(
            f"_execute method of {self.__class__.__name__} not implemented"
        )

    def execute(self, glyph) -> None:
        if self._checkParameters():
            self._execute(glyph)



"""
toolscale
===============================================================================
"""

from .base import WORKING, TransformBaseTool


class ToolScale(TransformBaseTool):
    def __init__(self, parent):
        super().__init__(parent)
        self.angleSnap = 45
        self.scalePrev = (1, 1)
        self.ratio = 100

    @property
    def scale(self):
        if self.state == WORKING:
            sx = 1 + (self.center[0] - self.canvas.screenToCanvasX(self.x)) / self.ratio
            sy = 1 + (self.center[1] - self.canvas.screenToCanvasY(self.y)) / self.ratio
            if self.shiftDown:
                a = abs(int(self.angle))
                if a in (0, 180):
                    sy = 1
                elif a in (90, 270):
                    sx = 1
                elif a in (45, 135, 225, 315):
                    sx = sy = (sx + sy) / 2
            return (sx, sy)
        return (1, 1)

    # -------------------------------------------------------------------------
    # Event handler methods
    # -------------------------------------------------------------------------

    def on_LEFT_UP(self, event):
        self.scalePrev = (1, 1)
        super().on_LEFT_UP(event)

    def on_MOTION(self, event):
        TransformBaseTool.on_MOTION(self, event)
        if event.Dragging() and self.state == WORKING:
            scale = self.scale
            delta = (self.scalePrev[0] - scale[0], self.scalePrev[1] - scale[1])
            if delta[0] or delta[1]:
                delta = (1 + delta[0], 1 + delta[1])
                glyph = self.glyph
                if self.isAnySelected():
                    for contour in glyph:
                        for point in contour:
                            if point.selected:
                                point.scaleBy(delta, self.center)
                                self.contoursChanged = True
                    for anchor in glyph.anchors:
                        if anchor.selected:
                            anchor.scaleBy(delta, self.center)
                            self.anchorsChanged = True
                    for component in glyph.components:
                        if component.selected:
                            component.scaleBy(delta, self.center)
                            self.componentsChanged = True
                else:
                    # todo: check locked plains
                    glyph.scaleBy(delta, self.center)
                self.canvas.Refresh()
            self.scalePrev = scale
        event.Skip()



"""
command
===============================================================================
"""
import logging
from functools import wraps

import wx
from booleanOperations import BooleanOperationManager as Boolean
from wbDefcon import Glyph
from wx import aui

from ..glyphCommand.commandListDialog import CommandListDialog
from ..view.font import UfoFontView
from ..view.glyph import UfoGlyphView

log = logging.getLogger(__name__)


def holdFontNotifications(func):
    """Decorator for glyph commands"""

    @wraps(func)
    def wrapper(*args, **kwds):
        glyph = None
        font = None
        if args:
            glyph = args[0]
        if not isinstance(glyph, Glyph):
            glyph = kwds.get("glyph")
        log.debug("holdFontNotifications %r", glyph)
        if glyph and glyph.font:
            font = glyph.font
            font.holdNotifications()
        result = func(*args, **kwds)
        if font:
            font.releaseHeldNotifications()
        return result

    return wrapper


@holdFontNotifications
def remove_overlap(glyph):
    contours = [c for c in glyph]
    glyph.disableNotifications()
    glyph.undoManager.saveState()
    Boolean.union([c for c in glyph], glyph.getPointPen())
    for contour in contours:
        glyph.removeContour(contour)
    glyph.round(roundAnchors=False, roundComponents=False)
    glyph.enableNotifications()
    glyph.postNotification("Glyph.ContoursChanged")


@holdFontNotifications
def contourDirectionPS(glyph):
    glyph.correctContourDirection()


@holdFontNotifications
def contourDirectionTT(glyph):
    glyph.correctContourDirection(trueType=True)


class GlyphCommandToolbar(aui.AuiToolBar):
    commands = (remove_overlap, contourDirectionPS, contourDirectionTT)

    def __init__(self, parent):
        id = wx.ID_ANY
        pos = wx.DefaultPosition
        size = wx.DefaultSize
        style = (
            wx.aui.AUI_TB_HORZ_LAYOUT | wx.aui.AUI_TB_PLAIN_BACKGROUND | wx.NO_BORDER
        )
        aui.AuiToolBar.__init__(self, parent, id, pos, size, style)
        self.SetToolBitmapSize(wx.Size(16, 16))
        self.appendTool("Remove Overlap", "MERGE_CONTOURS", "Merge contours", 0)
        self.AddSeparator()
        self.appendTool("Flip Horizontal", "FLIP_HORIZONTAL", "Flip horizontal")
        self.appendTool("Flip Vertical", "FLIP_VERTICAL", "Flip vertical")
        self.AddSeparator()
        self.appendTool(
            "Set PS Direction", "DIRECTION_PS", "Set PostScript contour direction", 1
        )
        self.appendTool(
            "Set TT Direction", "DIRECTION_TT", "Set TrueType contour direction", 2
        )
        self.AddSeparator()
        self.appendTool(
            "Command List",
            "COMMAND_LIST",
            "Execute Command List",
            handler=self.on_command_list,
        )

    @property
    def app(self):
        return wx.GetApp()

    @staticmethod
    def bitmap(name):
        return wx.ArtProvider.GetBitmap(name, wx.ART_TOOLBAR)

    def appendTool(self, label, bitmapName, helpText, commandIndex=-1, handler=None):
        if not handler:
            handler = self.on_Tool
        tool = self.AddTool(
            wx.ID_ANY, label, self.bitmap(bitmapName), helpText, wx.ITEM_NORMAL
        )
        tool.SetUserData(commandIndex)
        self.Bind(wx.EVT_TOOL, handler, tool)
        self.Bind(wx.EVT_UPDATE_UI, self.on_update_Tool, tool)
        return tool

    def on_Tool(self, event):
        tool = self.FindTool(event.Id)
        i = tool.GetUserData()
        if isinstance(i, int) and i >= 0:
            command = self.commands[i]
            view = self.app.documentManager.currentView
            if isinstance(view, UfoFontView):
                font = self.app.documentManager.currentView.font
                for glyph in font.selectedGlyphs:
                    command(glyph)
            elif isinstance(view, UfoGlyphView):
                command(view.glyph)
        else:
            label = self.GetToolLabel(event.Id)
            wx.LogWarning(f'Command "{label}" not yet implemented')

    def on_update_Tool(self, event):
        view = self.app.documentManager.currentView
        if view and view.document:
            if isinstance(view, UfoFontView):
                font = view.document.font
                if font and len(font.selectedGlyphNames) > 0:
                    event.Enable(True)
                else:
                    event.Enable(False)
            elif isinstance(view, UfoGlyphView):
                event.Enable(True)
            else:
                event.Enable(False)
        else:
            event.Enable(False)

    def on_command_list(self, event):
        with CommandListDialog() as dialog:
            view = self.app.documentManager.currentView
            if view and view.document:
                if isinstance(view, UfoFontView):
                    dialog.choice_target.SetSelection(1)  # selected glyphs
                elif isinstance(view, UfoGlyphView):
                    dialog.choice_target.SetSelection(0)  # current glyph
                if dialog.ShowModal() == wx.ID_OK:
                    commandList = dialog.commandList
                    if commandList:
                        dialog.executeCommandList()



"""
composite
===============================================================================

Glyph Commands related to composites
"""
# from glyphConstruction import (
#     GlyphConstructionBuilder,
#     ParseGlyphConstructionListFromString,
# )

from .base import GlyphCommand
from .parameter import ParamFilepathRequired


class DecomposeCommand(GlyphCommand):
    name = "Decompose"

    def _execute(self, glyph):
        glyph.decomposeAllComponents()


class ClearComponentsCommand(GlyphCommand):
    name = "Clear all components"

    def _execute(self, glyph):
        glyph.clearComponents()


# class UpdateCompositeCommand(GlyphCommand):
#     name = "Update composite"
#     parameters = [ParamFilepathRequired("c", "Glyph construction file", default="")]

#     def __init__(self, **kwargs):
#         super().__init__(**kwargs)
#         self._constructions = None

#     @property
#     def constructions(self):
#         if not self._constructions:
#             constructions = ParseGlyphConstructionListFromString(self.c)
#             self._constructions = dict(
#                 (c.split("=", 1)[0].strip(), c) for c in constructions
#             )
#         return self._constructions

#     def _execute(self, glyph):
#         if glyph.components and glyph.name in self.constructions:
#             name = glyph.name
#             font = glyph.font
#             constructionGlyph = GlyphConstructionBuilder(self.constructions[name], font)
#             # newGlyph = font.newGlyph(name)
#             glyph.clearComponents()
#             constructionGlyph.draw(glyph.getPen())
#             glyph.unicode = constructionGlyph.unicode
#             # newGlyph.note = constructionGlyph.note
#             glyph.width = constructionGlyph.width



# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.1-0-g8feb16b3)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class FontInfoEmbeddingUI
###########################################################################

class FontInfoEmbeddingUI ( wx.ScrolledWindow ):

    def __init__( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.HSCROLL|wx.VSCROLL, name = wx.EmptyString ):
        wx.ScrolledWindow.__init__ ( self, parent, id = id, pos = pos, size = size, style = style, name = name )

        self.SetExtraStyle( wx.WS_EX_VALIDATE_RECURSIVELY )
        self.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_WINDOW ) )

        sizerMain = wx.BoxSizer( wx.VERTICAL )

        boxEmbedding = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Embedding" ), wx.VERTICAL )

        sizerEmbedding = wx.GridBagSizer( 0, 0 )
        sizerEmbedding.SetFlexibleDirection( wx.HORIZONTAL )
        sizerEmbedding.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_openTypeOS2Type = wx.StaticText( self, wx.ID_ANY, u"Main", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeOS2Type.Wrap( -1 )

        self.lbl_openTypeOS2Type.SetMinSize( wx.Size( 170,-1 ) )

        sizerEmbedding.Add( self.lbl_openTypeOS2Type, wx.GBPosition( 0, 0 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.m_radioBtn2 = wx.RadioButton( self, wx.ID_ANY, u"No Embedding", wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2Type 2" )
        sizerEmbedding.Add( self.m_radioBtn2, wx.GBPosition( 0, 1 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.radioBtn_openTypeOS2Type3 = wx.RadioButton( self, wx.ID_ANY, u"Print and Preview Embedding", wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2Type 4" )
        sizerEmbedding.Add( self.radioBtn_openTypeOS2Type3, wx.GBPosition( 1, 1 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.m_radioBtn4 = wx.RadioButton( self, wx.ID_ANY, u"Editable Embedding", wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2Type 8" )
        sizerEmbedding.Add( self.m_radioBtn4, wx.GBPosition( 2, 1 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.m_radioBtn5 = wx.RadioButton( self, wx.ID_ANY, u"Installable Embedding", wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2Type 0" )
        sizerEmbedding.Add( self.m_radioBtn5, wx.GBPosition( 3, 1 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.lbl_postscriptUnderlineThickness = wx.StaticText( self, wx.ID_ANY, u"Options", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptUnderlineThickness.Wrap( -1 )

        sizerEmbedding.Add( self.lbl_postscriptUnderlineThickness, wx.GBPosition( 4, 0 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.checkBox_openTypeOS2Type_8 = wx.CheckBox( self, wx.ID_ANY, u"No Subsetting", wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2Type 8" )
        sizerEmbedding.Add( self.checkBox_openTypeOS2Type_8, wx.GBPosition( 4, 1 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )

        self.checkBox_openTypeOS2Type_9 = wx.CheckBox( self, wx.ID_ANY, u"Bitmap Embedding Only", wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2Type 9" )
        sizerEmbedding.Add( self.checkBox_openTypeOS2Type_9, wx.GBPosition( 5, 1 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )


        sizerEmbedding.AddGrowableCol( 1 )

        boxEmbedding.Add( sizerEmbedding, 1, wx.EXPAND, 5 )


        sizerMain.Add( boxEmbedding, 0, wx.ALL|wx.EXPAND, 5 )


        self.SetSizer( sizerMain )
        self.Layout()
        sizerMain.Fit( self )

    def __del__( self ):
        pass





# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.0-4761b0c)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class FontInfoClassificationUI
###########################################################################

class FontInfoClassificationUI ( wx.ScrolledWindow ):

    def __init__( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.HSCROLL|wx.VSCROLL, name = wx.EmptyString ):
        wx.ScrolledWindow.__init__ ( self, parent, id = id, pos = pos, size = size, style = style, name = name )

        self.SetExtraStyle( wx.WS_EX_VALIDATE_RECURSIVELY )
        self.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_WINDOW ) )

        sizerMain = wx.BoxSizer( wx.VERTICAL )

        boxPostScript = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"PostScript" ), wx.VERTICAL )

        sizerPostScript = wx.GridBagSizer( 0, 0 )
        sizerPostScript.SetFlexibleDirection( wx.HORIZONTAL )
        sizerPostScript.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_postscriptIsFixedPitch = wx.StaticText( self, wx.ID_ANY, u"Fixed Pitch", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptIsFixedPitch.Wrap( -1 )

        self.lbl_postscriptIsFixedPitch.SetMinSize( wx.Size( 170,-1 ) )

        sizerPostScript.Add( self.lbl_postscriptIsFixedPitch, wx.GBPosition( 0, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.checkBox_postscriptIsFixedPitch = wx.CheckBox( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"postscriptIsFixedPitch" )
        sizerPostScript.Add( self.checkBox_postscriptIsFixedPitch, wx.GBPosition( 0, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.EXPAND, 5 )


        sizerPostScript.AddGrowableCol( 1 )

        boxPostScript.Add( sizerPostScript, 1, wx.EXPAND, 5 )


        sizerMain.Add( boxPostScript, 0, wx.ALL|wx.EXPAND, 5 )

        boxOS2 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"OpenType OS/2 Table" ), wx.VERTICAL )

        sizerOS2 = wx.GridBagSizer( 0, 0 )
        sizerOS2.SetFlexibleDirection( wx.HORIZONTAL )
        sizerOS2.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_openTypeOS2WidthClass = wx.StaticText( self, wx.ID_ANY, u"Width Class", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeOS2WidthClass.Wrap( -1 )

        self.lbl_openTypeOS2WidthClass.SetMinSize( wx.Size( 170,-1 ) )

        sizerOS2.Add( self.lbl_openTypeOS2WidthClass, wx.GBPosition( 0, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeOS2WidthClass = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2WidthClass" )
        sizerOS2.Add( self.txtCtrl_openTypeOS2WidthClass, wx.GBPosition( 0, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_openTypeOS2WeightClass = wx.StaticText( self, wx.ID_ANY, u"Weight Class", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeOS2WeightClass.Wrap( -1 )

        sizerOS2.Add( self.lbl_openTypeOS2WeightClass, wx.GBPosition( 1, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeOS2WeightClass = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2WeightClass" )
        sizerOS2.Add( self.txtCtrl_openTypeOS2WeightClass, wx.GBPosition( 1, 1 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )


        sizerOS2.AddGrowableCol( 1 )

        boxOS2.Add( sizerOS2, 1, wx.EXPAND, 5 )


        sizerMain.Add( boxOS2, 0, wx.ALL|wx.EXPAND, 5 )


        self.SetSizer( sizerMain )
        self.Layout()
        sizerMain.Fit( self )

    def __del__( self ):
        pass





import wx

# from wbDefcon import Glyph
# from wbFontParts import RGlyph

from .glyphViewPanelUI import GlyphViewPanelUI

name = "GlyphViewPanel"


class GlyphViewPanel(GlyphViewPanelUI):
    def __init__(
        self,
        parent,
        id=wx.ID_ANY,
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=wx.TAB_TRAVERSAL,
        name=name,
    ):
        super().__init__(parent, id=id, pos=pos, size=size, style=style, name=name)

    @property
    def fontsize(self):
        return self.spinCtrl_font_size.Value

    @fontsize.setter
    def fontsize(self, value):
        self.spinCtrl_font_size.Value = value
        self.glyphViewCanvas.Refresh()

    @property
    def linespace(self):
        return self.spinCtrl_line_space.Value

    @property
    def glyphs(self):
        return self.glyphViewCanvas.glyphs

    def insertGlyph(self, index, glyph):
        self.glyphViewCanvas.insertGlyph(index, glyph)

    def appendGlyph(self, glyph):
        self.glyphViewCanvas.insertGlyph(self.glyphViewCanvas.glyphCount, glyph)

    def clearGlyphs(self):
        self.glyphViewCanvas.clearGlyphs()

    # =================================================================
    # event handlers
    # =================================================================
    def on_spinCtrl_font_size(self, event):
        self.glyphViewCanvas.Refresh()

    def on_spinCtrl_line_space(self, event):
        self.glyphViewCanvas.Refresh()

    def on_checkBox_showKerning(self, event):
        self.glyphViewCanvas.Refresh()

    def on_btn_clear(self, event):
        self.clearGlyphs()


glyphViewPanelInfo = wx.aui.AuiPaneInfo()
glyphViewPanelInfo.Name(name)
glyphViewPanelInfo.Caption(name)
glyphViewPanelInfo.Dock()
glyphViewPanelInfo.Bottom()
glyphViewPanelInfo.Resizable()
glyphViewPanelInfo.MaximizeButton(True)
glyphViewPanelInfo.MinimizeButton(True)
glyphViewPanelInfo.CloseButton(False)
glyphViewPanelInfo.FloatingSize(wx.Size(300, 200))
glyphViewPanelInfo.BestSize(wx.Size(800, 400))
glyphViewPanelInfo.MinSize(wx.Size(300, 100))
glyphViewPanelInfo.Icon(wx.ArtProvider.GetBitmap("VIEW", wx.ART_FRAME_ICON))



"""
selectFontsDialog
===============================================================================
"""
import wx
from .. import CurrentFont, AllFonts

from .selectFontsDialogUI import SelectFontsDialogUI


class SelectFontsDialog(SelectFontsDialogUI):
    def __init__(self, message="Select fonts:", title="UFO Workbench", allFonts=None):
        if allFonts is None:
            allFonts = AllFonts()
        self.allFonts = allFonts
        super().__init__(wx.GetApp().TopWindow)
        self.Title = title
        self.label_message.Label = message
        self.sdbSizerOK.SetDefault()
        fontList = self.listCtrl_fonts
        currentFont = CurrentFont()
        for i in range(fontList.ItemCount):
            if fontList.allFonts[i] == currentFont:
                fontList.SetItemState(i, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)
        
    @property
    def selectedFonts(self):
        return self.listCtrl_fonts.getSelectedFonts()

    def on_KEY_DOWN(self, event):
        unicodeKey = event.GetUnicodeKey()
        if unicodeKey != wx.WXK_NONE:
            key = chr(unicodeKey)
            if key == "A" and (event.ControlDown() or event.CmdDown()):
                fontList = self.listCtrl_fonts
                for i in range(fontList.ItemCount):
                    fontList.SetItemState(
                        i, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED
                    )
                self.sdbSizerOK.SetFocus()
            else:
                event.Skip()
        else:
            event.Skip()




"""
font
===============================================================================
"""
import logging

import wx

from wbBase.tools import get_wxBrush, get_wxPen

from .base import DrawingPlane, DrawingPlaneStack, GuidelinePlaneMixin

log = logging.getLogger(__name__)


class FontLevelPlain(DrawingPlane):
    """
    Base class for all font level drawing planes
    """

    visibleDflt = True

    def __init__(self, parent, name):
        super().__init__(parent, name)
        self._visible = self.visibleDflt

    @property
    def visible(self):
        return self._visible

    @visible.setter
    def visible(self, value):
        newVal = bool(value)
        if newVal != self._visible:
            self._visible = newVal
            self.canvas.Refresh()


# -------------------------------------------------------------------------
# Font level plains
# -------------------------------------------------------------------------


class VerticalMetricPlane(FontLevelPlain):
    """
    Font level metrics like, ascender, descender ...
    """

    pen = get_wxPen(color="gray", style=wx.PENSTYLE_LONG_DASH)

    def __init__(self, parent):
        super().__init__(parent, "VerticalMetric")

    def draw(self, gc):
        gc.SetPen(self.pen)
        transformPoint = self.transform.transformPoint
        w = gc.GetSize()[0]
        info = self.font.info
        for h in (info.ascender, info.capHeight, info.xHeight, 0, info.descender):
            if h is not None:
                y = round(transformPoint((0, h))[1])
                gc.StrokeLine(0, y, w, y)


class AlignmentZonesPlane(FontLevelPlain):
    """
    Draw PostScript alignment zones of the font
    """

    visibleDflt = False
    pen = get_wxPen(color=wx.Colour(0, 0, 255, 128), style=wx.PENSTYLE_LONG_DASH)
    brush = get_wxBrush(color=wx.Colour(0, 0, 255, 32))

    def __init__(self, parent):
        super().__init__(parent, "AlignmentZones")

    def draw(self, gc):
        gc.SetPen(self.pen)
        gc.SetBrush(self.brush)
        transformPoint = self.transform.transformPoint
        w = gc.GetSize()[0] + 1
        x = -1
        blueValues = (
            self.font.info.postscriptBlueValues + self.font.info.postscriptOtherBlues
        )
        for bottom, top in [
            (blueValues[i], blueValues[i + 1]) for i in range(0, len(blueValues), 2)
        ]:
            y = round(transformPoint((0, top))[1])
            h = abs(y - round(transformPoint((0, bottom))[1]))
            log.debug("y = %s, h = %s", y, h)
            gc.DrawRectangle(x, y, w, h)


class FontGuidePlane(FontLevelPlain, GuidelinePlaneMixin):
    """
    Draw guidelines of font on Canvas
    """

    selected_pen = get_wxPen(
        color=wx.Colour(255, 0, 0, 64), width=3, style=wx.PENSTYLE_SOLID
    )

    def __init__(self, parent):
        super().__init__(parent, "FontGuide")

    def draw(self, gc):
        self.drawGuidelines(gc, self.font.guidelines)

    def hitTest(self, x, y):
        return self.hitTestGuidelines(x, y, self.font.guidelines)


class FontLevelPlanes(DrawingPlaneStack):
    """
    Collection of font level drawing planes
    """

    childType = FontLevelPlain
    visible = True

    def __init__(self, parent):
        super().__init__(parent, "Font")
        self.addPlane(VerticalMetricPlane)
        self.addPlane(AlignmentZonesPlane)
        self.addPlane(FontGuidePlane)



"""
layer
===============================================================================

Glyph Commands related to layers
"""
import logging

from wbDefcon.objects.glyph import Glyph

from .base import GlyphCommand
from .parameter import ParamEditableEnumeration

log = logging.getLogger(__name__)


class CopyToLayer(GlyphCommand):
    name = "Copy from default Layer to other Layer"
    parameters = [
        ParamEditableEnumeration(
            "t",
            "Target Layer",
            ["public.background", "private.temp"],
            default="public.background",
        )
    ]

    def _execute(self, glyph):
        font = glyph.font
        if self.t in font.layers:
            targetLayer = font.layers[self.t]
        else:
            targetLayer = font.newLayer(self.t)
        if glyph.name in targetLayer:
            targetGlyph = targetLayer[glyph.name]
        else:
            targetGlyph = targetLayer.newGlyph(glyph.name)
        targetGlyph.copyDataFromGlyph(glyph)


class CopyFromLayer(GlyphCommand):
    name = "Copy from other Layer to default Layer"
    parameters = [
        ParamEditableEnumeration(
            "s", "Source Layer", ["public.background"], default="public.background"
        )
    ]

    def _execute(self, glyph):
        font = glyph.font
        if self.s in font.layers:
            sourceLayer = font.layers[self.s]
            if glyph.name in sourceLayer:
                sourceGlyph = sourceLayer[glyph.name]
                glyph.copyDataFromGlyph(sourceGlyph)
            else:
                log.warning("Glyph %s not found on layer %r", glyph.name, self.s)
        else:
            log.warning("Layer %r for glyph %s does not exist", self.s, glyph.name)


class ExchangeWithLayer(GlyphCommand):
    name = "Exchange default Layer with other Layer"
    parameters = [
        ParamEditableEnumeration(
            "o", "Other Layer", ["public.background"], default="public.background"
        )
    ]

    def _execute(self, glyph):
        font = glyph.font
        name = glyph.name
        if self.o in font.layers:
            targetLayer = font.layers[self.o]
        else:
            targetLayer = font.newLayer(self.o)
        # move glyph from targetLayer to tempGlyph
        tempGlyph = None
        if name in targetLayer:
            tempGlyph = Glyph()
            targetGlyph = targetLayer[name]
            tempGlyph.copyDataFromGlyph(targetGlyph)
            targetGlyph.clear()
        else:
            targetGlyph = targetLayer.newGlyph(name)
        # move glyph from default to targetLayer
        targetGlyph.copyDataFromGlyph(glyph)
        glyph.clear()
        # move glyph from tempGlyph to active layer
        if tempGlyph is not None:
            glyph.copyDataFromGlyph(tempGlyph)


class ClearLayer(GlyphCommand):
    name = "Clear Layer"
    parameters = [
        ParamEditableEnumeration(
            "l", "Layer", ["public.background"], default="public.background"
        )
    ]

    def _execute(self, glyph):
        font = glyph.font
        if self.l in font.layers:
            layer = font.layers[self.l]
            if glyph.name in layer:
                del layer[glyph.name]
                if len(layer) == 0:
                    del font.layers[self.l]



"""
panelItemPicker
===============================================================================
"""
import wx

from .panelItemPickerUI import PanelItemPickerUI


class PanelItemPicker(PanelItemPickerUI):
    def __init__(
        self,
        parent,
        id=wx.ID_ANY,
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=wx.TAB_TRAVERSAL,
    ):
        PanelItemPickerUI.__init__(
            self, parent, id=id, pos=pos, size=size, style=style, name="PanelItemPicker"
        )

    @property
    def options(self):
        return self.listBox_options.Items

    @options.setter
    def options(self, value):
        self.listBox_options.Items = value

    @property
    def selection(self):
        return self.listBox_selection.Items

    @selection.setter
    def selection(self, value):
        self.listBox_selection.Items = value

    def _moveItems(self, source, dest):
        selections = source.GetSelections()
        selectedItems = list(map(source.GetString, selections))
        dest.SetItems(dest.GetItems() + selectedItems)
        selections = set(selections)
        source.SetItems(
            [item for i, item in enumerate(source.GetItems()) if i not in selections]
        )

    # -------------------------------------------------------------------------
    # Event handler methods
    # -------------------------------------------------------------------------

    def on_listBox_dblclick(self, event):
        listBox = event.GetEventObject()
        selections = listBox.GetSelections()
        if len(selections) != 1:
            return  # DCLICK only works on one item
        if event.GetSelection() != selections[0]:
            # this can happen using ^DCLICK when two items are selected
            return
        if listBox == self.listBox_options:
            self.on_select(event)
        else:
            self.on_unselect(event)

    def on_select(self, event):
        self._moveItems(self.listBox_options, self.listBox_selection)

    def update_button_select(self, event):
        event.Enable(len(self.listBox_options.Selections) > 0)

    def on_unselect(self, event):
       self._moveItems(self.listBox_selection, self.listBox_options)

    def update_button_unselect(self, event):
        event.Enable(len(self.listBox_selection.Selections) > 0)

    def on_button_move_up(self, event):
        event.Skip()

    def update_button_move_up(self, event):
        event.Skip()

    def on_button_move_down(self, event):
        event.Skip()

    def update_button_move_down(self, event):
        event.Skip()

    def on_button_clear(self, event):
        self.options = self.options + self.selection
        self.selection = []

    def update_button_clear(self, event):
        event.Enable(len(self.selection)>0)




"""
guideline
===============================================================================

Glyph Commands related to guidelines
"""
import logging

import wx
from wbDefcon import Color, Guideline

from .base import GlyphCommand
from .parameter import (
    ParamBoolRequired,
    ParamColour,
    ParamFloatRequired,
    ParamIntRequired,
    ParamStrRequired,
)

log = logging.getLogger(__name__)


class ClearGuides(GlyphCommand):
    name = "Clear all local guides"

    def _execute(self, glyph):
        glyph.clearGuidelines()


class AddGuidelineCommand(GlyphCommand):
    name = "Add local guideline"
    parameters = [
        ParamIntRequired("x", "Horizontal position (x)", 0),
        ParamIntRequired("y", "Vertical position (y)", 0),
        ParamFloatRequired("a", "Angle", 0.0),
        ParamStrRequired("n", "Name"),
        ParamColour("c", "Colour", wx.WHITE),
        ParamBoolRequired("r", "Replace existing guideline (by name)", True),
    ]

    def _execute(self, glyph):
        if self.n and self.r:
            for guideline in glyph.guidelines:
                if guideline.name == self.n:
                    guideline.x = self.x
                    guideline.y = self.y
                    guideline.angle = self.a
                    guideline.name = self.n
                    if self.c == wx.WHITE:
                        guideline.color = None
                    else:
                        guideline.color = Color.from_wx(self.c)
                    return
        guideline = Guideline(guidelineDict=dict(x=self.x, y=self.y, angle=self.a))
        if self.n:
            guideline.name = self.n
        if self.c != wx.WHITE:
            guideline.color = Color.from_wx(self.c)
        glyph.appendGuideline(guideline)


class RemoveGuidelineCommand(GlyphCommand):
    name = "Remove guideline (by name)"
    parameters = [ParamStrRequired("n", "Name")]

    def _execute(self, glyph):
        for guideline in reversed(glyph.guidelines):
            if guideline.name == self.n:
                glyph.removeGuideline(anchor)



import wx
from wx import aui


class BaseToolbar(aui.AuiToolBar):
    itemType = wx.ITEM_NORMAL
    def __init__(self, parent, name=None):
        id = wx.ID_ANY
        pos = wx.DefaultPosition
        size = wx.DefaultSize
        style = (
            wx.aui.AUI_TB_HORZ_LAYOUT | wx.aui.AUI_TB_PLAIN_BACKGROUND | wx.NO_BORDER
        )
        super().__init__(parent, id, pos, size, style)
        if isinstance(name, str):
            self.Name = name
        self.SetToolBitmapSize(wx.Size(16, 16))

    @property
    def app(self):
        return wx.GetApp()

    @property
    def currentView(self):
        return self.app.documentManager.currentView

    @staticmethod
    def bitmap(name):
        return wx.ArtProvider.GetBitmap(name, wx.ART_TOOLBAR)

    def appendTool(
        self, label, bitmapName, helpText=wx.EmptyString, commandIndex=-1, kind=None
    ):
        if not helpText:
            helpText = label
        if kind is None:
            kind = self.itemType
        tool = self.AddTool(
            toolId=wx.ID_ANY,
            label=label,
            bitmap=self.bitmap(bitmapName),
            short_help_string=helpText,
            kind=kind,
        )
        tool.SetUserData(commandIndex)
        self.Bind(wx.EVT_TOOL, self.on_Tool, tool)
        self.Bind(wx.EVT_UPDATE_UI, self.on_update_Tool, tool)
        return tool

    def on_Tool(self, event):
        event.Skip()

    def on_update_Tool(self, event):
        event.Skip()



# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.1-0-g8feb16b3)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class FontInfoMetricUI
###########################################################################

class FontInfoMetricUI ( wx.ScrolledWindow ):

    def __init__( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.HSCROLL|wx.VSCROLL, name = wx.EmptyString ):
        wx.ScrolledWindow.__init__ ( self, parent, id = id, pos = pos, size = size, style = style, name = name )

        self.SetExtraStyle( wx.WS_EX_VALIDATE_RECURSIVELY )
        self.SetBackgroundColour( wx.Colour( 255, 255, 255 ) )

        sizer = wx.BoxSizer( wx.VERTICAL )

        boxMain = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Main" ), wx.VERTICAL )

        gridMain = wx.FlexGridSizer( 0, 2, 0, 0 )
        gridMain.AddGrowableCol( 1 )
        gridMain.SetFlexibleDirection( wx.HORIZONTAL )
        gridMain.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_unitsPerEm = wx.StaticText( self, wx.ID_ANY, u"Units per Em", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_unitsPerEm.Wrap( -1 )

        self.lbl_unitsPerEm.SetMinSize( wx.Size( 170,-1 ) )

        gridMain.Add( self.lbl_unitsPerEm, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_unitsPerEm = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"unitsPerEm" )
        gridMain.Add( self.txtCtrl_unitsPerEm, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_ascender = wx.StaticText( self, wx.ID_ANY, u"Ascender", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_ascender.Wrap( -1 )

        gridMain.Add( self.lbl_ascender, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_ascender = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"ascender" )
        gridMain.Add( self.txtCtrl_ascender, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_capHeight = wx.StaticText( self, wx.ID_ANY, u"Cap-Height", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_capHeight.Wrap( -1 )

        gridMain.Add( self.lbl_capHeight, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_capHeight = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"capHeight" )
        gridMain.Add( self.txtCtrl_capHeight, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_xHeight = wx.StaticText( self, wx.ID_ANY, u"x-Height", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_xHeight.Wrap( -1 )

        gridMain.Add( self.lbl_xHeight, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_xHeight = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"xHeight" )
        gridMain.Add( self.txtCtrl_xHeight, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_descender = wx.StaticText( self, wx.ID_ANY, u"Descender", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_descender.Wrap( -1 )

        gridMain.Add( self.lbl_descender, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_descender = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"descender" )
        gridMain.Add( self.txtCtrl_descender, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_italicAngle = wx.StaticText( self, wx.ID_ANY, u"Italic Angle", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_italicAngle.Wrap( -1 )

        gridMain.Add( self.lbl_italicAngle, 0, wx.ALL, 5 )

        self.txtCtrl_italicAngle = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"italicAngle" )
        gridMain.Add( self.txtCtrl_italicAngle, 0, wx.ALL, 5 )


        boxMain.Add( gridMain, 0, wx.EXPAND, 5 )


        sizer.Add( boxMain, 0, wx.ALL|wx.EXPAND, 5 )

        boxHhea = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"hhea Metric" ), wx.VERTICAL )

        fgSizer2 = wx.FlexGridSizer( 0, 2, 0, 0 )
        fgSizer2.AddGrowableCol( 1 )
        fgSizer2.SetFlexibleDirection( wx.HORIZONTAL )
        fgSizer2.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_openTypeHheaAscender = wx.StaticText( self, wx.ID_ANY, u"Ascender", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeHheaAscender.Wrap( -1 )

        self.lbl_openTypeHheaAscender.SetMinSize( wx.Size( 170,-1 ) )

        fgSizer2.Add( self.lbl_openTypeHheaAscender, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeHheaAscender = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeHheaAscender" )
        fgSizer2.Add( self.txtCtrl_openTypeHheaAscender, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_openTypeHheaDescender = wx.StaticText( self, wx.ID_ANY, u"Descender", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeHheaDescender.Wrap( -1 )

        fgSizer2.Add( self.lbl_openTypeHheaDescender, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeHheaDescender = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeHheaDescender" )
        fgSizer2.Add( self.txtCtrl_openTypeHheaDescender, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_openTypeHheaLineGap = wx.StaticText( self, wx.ID_ANY, u"Line Gap", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeHheaLineGap.Wrap( -1 )

        fgSizer2.Add( self.lbl_openTypeHheaLineGap, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeHheaLineGap = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeHheaLineGap" )
        fgSizer2.Add( self.txtCtrl_openTypeHheaLineGap, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


        boxHhea.Add( fgSizer2, 1, wx.EXPAND, 5 )


        sizer.Add( boxHhea, 0, wx.ALL|wx.EXPAND, 5 )

        boxOS2typo = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"OS/2 Typo Metric" ), wx.VERTICAL )

        gridOS2typo = wx.FlexGridSizer( 0, 2, 0, 0 )
        gridOS2typo.SetFlexibleDirection( wx.HORIZONTAL )
        gridOS2typo.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_openTypeOS2TypoAscender = wx.StaticText( self, wx.ID_ANY, u"Ascender", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeOS2TypoAscender.Wrap( -1 )

        self.lbl_openTypeOS2TypoAscender.SetMinSize( wx.Size( 170,-1 ) )

        gridOS2typo.Add( self.lbl_openTypeOS2TypoAscender, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeOS2TypoAscender = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2TypoAscender" )
        gridOS2typo.Add( self.txtCtrl_openTypeOS2TypoAscender, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_openTypeOS2TypoDescender = wx.StaticText( self, wx.ID_ANY, u"Descender", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeOS2TypoDescender.Wrap( -1 )

        gridOS2typo.Add( self.lbl_openTypeOS2TypoDescender, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeOS2TypoDescender = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2TypoDescender" )
        gridOS2typo.Add( self.txtCtrl_openTypeOS2TypoDescender, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_openTypeOS2TypoLineGap = wx.StaticText( self, wx.ID_ANY, u"Line Gap", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeOS2TypoLineGap.Wrap( -1 )

        gridOS2typo.Add( self.lbl_openTypeOS2TypoLineGap, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeOS2TypoLineGap = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2TypoLineGap" )
        gridOS2typo.Add( self.txtCtrl_openTypeOS2TypoLineGap, 0, wx.ALL, 5 )

        self.m_staticText16 = wx.StaticText( self, wx.ID_ANY, u"Selection Bit 7", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.m_staticText16.Wrap( -1 )

        gridOS2typo.Add( self.m_staticText16, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.checkBox_openTypeOS2Selection_7 = wx.CheckBox( self, wx.ID_ANY, u"Use Typo Metric", wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2Selection 7" )
        gridOS2typo.Add( self.checkBox_openTypeOS2Selection_7, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


        boxOS2typo.Add( gridOS2typo, 1, wx.EXPAND, 5 )


        sizer.Add( boxOS2typo, 0, wx.ALL|wx.EXPAND, 5 )

        boxOS2win = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"OS/2 Win Metric" ), wx.VERTICAL )

        gridOS2win = wx.FlexGridSizer( 0, 2, 0, 0 )
        gridOS2win.SetFlexibleDirection( wx.HORIZONTAL )
        gridOS2win.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_openTypeOS2WinAscent = wx.StaticText( boxOS2win.GetStaticBox(), wx.ID_ANY, u"Ascender", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeOS2WinAscent.Wrap( -1 )

        self.lbl_openTypeOS2WinAscent.SetMinSize( wx.Size( 170,-1 ) )

        gridOS2win.Add( self.lbl_openTypeOS2WinAscent, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeOS2WinAscent = wx.TextCtrl( boxOS2win.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2WinAscent" )
        gridOS2win.Add( self.txtCtrl_openTypeOS2WinAscent, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_openTypeOS2WinDescent = wx.StaticText( boxOS2win.GetStaticBox(), wx.ID_ANY, u"Descender", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeOS2WinDescent.Wrap( -1 )

        gridOS2win.Add( self.lbl_openTypeOS2WinDescent, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeOS2WinDescent = wx.TextCtrl( boxOS2win.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2WinDescent" )
        gridOS2win.Add( self.txtCtrl_openTypeOS2WinDescent, 0, wx.ALL, 5 )


        boxOS2win.Add( gridOS2win, 1, wx.EXPAND, 5 )


        sizer.Add( boxOS2win, 0, wx.ALL|wx.EXPAND, 5 )

        boxVhea = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"vhea Typo Metric" ), wx.VERTICAL )

        gridVhea = wx.FlexGridSizer( 0, 2, 0, 0 )
        gridVhea.AddGrowableCol( 1 )
        gridVhea.SetFlexibleDirection( wx.HORIZONTAL )
        gridVhea.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_openTypeVheaVertTypoAscender = wx.StaticText( boxVhea.GetStaticBox(), wx.ID_ANY, u"Vertical Ascender", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeVheaVertTypoAscender.Wrap( -1 )

        self.lbl_openTypeVheaVertTypoAscender.SetMinSize( wx.Size( 170,-1 ) )

        gridVhea.Add( self.lbl_openTypeVheaVertTypoAscender, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeVheaVertTypoAscender = wx.TextCtrl( boxVhea.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeVheaVertTypoAscender" )
        gridVhea.Add( self.txtCtrl_openTypeVheaVertTypoAscender, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_openTypeVheaVertTypoDescender = wx.StaticText( boxVhea.GetStaticBox(), wx.ID_ANY, u"Vertical Descender", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeVheaVertTypoDescender.Wrap( -1 )

        gridVhea.Add( self.lbl_openTypeVheaVertTypoDescender, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeVheaVertTypoDescender = wx.TextCtrl( boxVhea.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeVheaVertTypoDescender" )
        gridVhea.Add( self.txtCtrl_openTypeVheaVertTypoDescender, 0, wx.ALL, 5 )

        self.lbl_openTypeVheaVertTypoLineGap = wx.StaticText( boxVhea.GetStaticBox(), wx.ID_ANY, u"Vertical LineGap", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeVheaVertTypoLineGap.Wrap( -1 )

        gridVhea.Add( self.lbl_openTypeVheaVertTypoLineGap, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeVheaVertTypoLineGap = wx.TextCtrl( boxVhea.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeVheaVertTypoLineGap" )
        gridVhea.Add( self.txtCtrl_openTypeVheaVertTypoLineGap, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


        boxVhea.Add( gridVhea, 1, wx.EXPAND, 5 )


        sizer.Add( boxVhea, 0, wx.ALL|wx.EXPAND, 5 )


        self.SetSizer( sizer )
        self.Layout()
        sizer.Fit( self )

    def __del__( self ):
        pass





# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version Oct 26 2018)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class ApplyGlyphConstructionDialogUI
###########################################################################

class ApplyGlyphConstructionDialogUI ( wx.Dialog ):

    def __init__( self, parent ):
        wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Apply Glyph-Construction", pos = wx.DefaultPosition, size = wx.Size( 450,300 ), style = wx.CAPTION|wx.RESIZE_BORDER )

        self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )

        sizer = wx.BoxSizer( wx.VERTICAL )

        self.label = wx.StaticText( self, wx.ID_ANY, u"Select Gylph-Construction File", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.label.Wrap( -1 )

        sizer.Add( self.label, 0, wx.ALL, 5 )

        self.filePicker_ConstructionPath = wx.FilePickerCtrl( self, wx.ID_ANY, wx.EmptyString, u"Select Gylph-Construction File", u"*.glyphConstruction", wx.DefaultPosition, wx.DefaultSize, wx.FLP_FILE_MUST_EXIST|wx.FLP_OPEN|wx.FLP_SMALL|wx.FLP_USE_TEXTCTRL )
        sizer.Add( self.filePicker_ConstructionPath, 0, wx.ALL|wx.EXPAND, 5 )

        sizerOptions = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Options" ), wx.VERTICAL )

        self.checkBox_NewGlyphs = wx.CheckBox( self, wx.ID_ANY, u"Create New Glyphs", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.checkBox_NewGlyphs.SetValue(True)
        sizerOptions.Add( self.checkBox_NewGlyphs, 0, wx.ALL|wx.EXPAND, 5 )

        self.checkBox_ReplaceOutline = wx.CheckBox( self, wx.ID_ANY, u"Replace Existing Outline", wx.DefaultPosition, wx.DefaultSize, 0 )
        sizerOptions.Add( self.checkBox_ReplaceOutline, 0, wx.ALL|wx.EXPAND, 5 )

        sizerSave = wx.BoxSizer( wx.HORIZONTAL )


        sizerSave.Add( ( 20, 0), 0, wx.EXPAND, 5 )

        self.checkBox_SaveOutline = wx.CheckBox( self, wx.ID_ANY, u"Save Existing Outline on Background Layer", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.checkBox_SaveOutline.Enable( False )

        sizerSave.Add( self.checkBox_SaveOutline, 0, wx.ALL, 5 )


        sizerOptions.Add( sizerSave, 0, wx.EXPAND, 0 )

        self.checkBox_ReplaceComposite = wx.CheckBox( self, wx.ID_ANY, u"Replace Existing Composites", wx.DefaultPosition, wx.DefaultSize, 0 )
        sizerOptions.Add( self.checkBox_ReplaceComposite, 0, wx.ALL|wx.EXPAND, 5 )

        sizerMark = wx.BoxSizer( wx.HORIZONTAL )

        self.checkBox_Mark = wx.CheckBox( self, wx.ID_ANY, u"Mark", wx.DefaultPosition, wx.DefaultSize, 0 )
        sizerMark.Add( self.checkBox_Mark, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.colourPicker_Mark = wx.ColourPickerCtrl( self, wx.ID_ANY, wx.BLACK, wx.DefaultPosition, wx.DefaultSize, wx.CLRP_DEFAULT_STYLE )
        self.colourPicker_Mark.Enable( False )

        sizerMark.Add( self.colourPicker_Mark, 0, wx.ALIGN_CENTER_VERTICAL, 0 )


        sizerOptions.Add( sizerMark, 1, wx.EXPAND, 5 )


        sizer.Add( sizerOptions, 1, wx.ALL|wx.EXPAND, 5 )

        buttons = wx.StdDialogButtonSizer()
        self.buttonsOK = wx.Button( self, wx.ID_OK )
        buttons.AddButton( self.buttonsOK )
        self.buttonsCancel = wx.Button( self, wx.ID_CANCEL )
        buttons.AddButton( self.buttonsCancel )
        buttons.Realize();

        sizer.Add( buttons, 1, wx.ALL|wx.EXPAND, 5 )


        self.SetSizer( sizer )
        self.Layout()

        self.Centre( wx.BOTH )

        # Connect Events
        self.checkBox_SaveOutline.Bind( wx.EVT_UPDATE_UI, self.update_SaveOutline )
        self.colourPicker_Mark.Bind( wx.EVT_UPDATE_UI, self.update_Mark )

    def __del__( self ):
        pass


    # Virtual event handlers, overide them in your derived class
    def update_SaveOutline( self, event ):
        event.Skip()

    def update_Mark( self, event ):
        event.Skip()





"""
base
===============================================================================

Base classes related to drawing on the cnavas of the glyph editor
"""

import logging
from math import cos, radians, sin

import wx
from fontTools.misc.transform import Identity
from wbBase.tools import get_wxBrush, get_wxFont, get_wxPen

log = logging.getLogger(__name__)

try:
    from math import dist
except ImportError:
    # backport for python 3.7
    from math import sqrt

    def dist(p0, p1):
        dx = abs(p0[0] - p1[0])
        dy = abs(p0[1] - p1[1])
        return sqrt(dx**2 + dy**2)


# -------------------------------------------------------------------------
# Base classes
# -------------------------------------------------------------------------


class BasePlane:
    """
    Base class for all Plain objects
    """

    def __init__(self, parent, name):
        self.parent = parent
        self.name = name
        self.canvas = None

    def __repr__(self):
        return f'<{self.__class__.__name__}: "{self.name}">'

    @property
    def transform(self):
        return self.parent.transform

    def draw(self, gc):
        """
        should be overridden by subclass
        """
        raise NotImplementedError


class DrawingPlane(BasePlane):
    """
    Base class to represent DrawingPlane on Canvas
    """

    lockedDflt = False

    def __init__(self, parent, name):
        super(DrawingPlane, self).__init__(parent, name)
        self._locked = self.lockedDflt

    @property
    def locked(self):
        return self._locked

    @locked.setter
    def locked(self, value):
        self._locked = bool(value)

    @property
    def font(self):
        return self.parent.parent.font

    def getInactiveColour(self, colour):
        return wx.Colour(
            colour.red, colour.green, colour.blue, self.parent.inactiveAlpha
        )

    def getInactiveBrush(self, brush):
        return get_wxBrush(self.getInactiveColour(brush.Colour), brush.Style)

    def getInactivePen(self, pen):
        return get_wxPen(self.getInactiveColour(pen.Colour), pen.Width, pen.Style)

    def hitTest(self, x, y):
        """
        may be overridden by subclass
        Coordinates x, y are in canvas/font units
        """
        return None

    def Refresh(self):
        self.parent.Refresh()


class DrawingPlaneStack(BasePlane):
    """
    Collection of DrawingPlane objects
    """

    childType = DrawingPlane
    activeDflt = False
    inactiveAlphaDflt = 128

    def __init__(self, parent, name):
        super().__init__(parent, name)
        self._active = self.activeDflt
        self._inactiveAlpha = self.inactiveAlphaDflt
        self.canvas = parent
        self._drawingPlanes = []
        self._hitTestOrder = None

    def __iter__(self):
        return iter(self._drawingPlanes)

    def __getitem__(self, key):
        for plane in self._drawingPlanes:
            if plane.name == key:
                return plane
        raise KeyError

    @property
    def active(self):
        return self._active

    @active.setter
    def active(self, value):
        newVal = bool(value)
        if newVal != self._active:
            self._active = newVal
            self.canvas.Refresh()

    @property
    def inactiveAlpha(self):
        return self._inactiveAlpha

    @property
    def hitTestOrder(self):
        if self._hitTestOrder is None:
            return [p.name for p in self._drawingPlanes]
        return self._hitTestOrder

    def addPlane(self, plane):
        assert issubclass(plane, self.childType)
        newPlane = plane(self)
        newPlane.canvas = self.parent
        self._drawingPlanes.append(newPlane)

    def keys(self):
        return [p.name for p in self._drawingPlanes]

    def get(self, name, default=None):
        for plane in self._drawingPlanes:
            if plane.name == name:
                return plane
        return default

    def draw(self, gc):
        for plane in [p for p in self._drawingPlanes if p.visible]:
            plane.draw(gc)


class GuidelinePlaneMixin:
    """
    Mixin class for font and glyph level guidelines
    """

    LabelFont = get_wxFont(pointSize=7, faceName="Small Fonts")
    LabelPen = wx.TRANSPARENT_PEN
    LabelBrush = get_wxBrush(wx.Colour(230, 230, 240, 200))

    def drawGuidelines(self, gc, guidelines):
        gc.SetFont(self.LabelFont, wx.BLACK)
        d = 3  # distace between point and label
        s = 2  # space around text
        for guideline in guidelines:
            gc.SetBrush(wx.TRANSPARENT_BRUSH)
            line = ((-8000, 0), (8000, 0))
            if guideline.angle is None:
                if guideline.x is None:
                    guideline_x = 0
                    guideline_y = guideline.y
                    guideline_angle = 0
                elif guideline.y is None:
                    guideline_x = guideline.x
                    guideline_y = 0
                    guideline_angle = 90
            else:
                guideline_x = guideline.x
                guideline_y = guideline.y
                guideline_angle = guideline.angle
            line = (
                Identity.translate(guideline_x, guideline_y)
                .rotate(radians(guideline_angle))
                .transformPoints(line)
            )
            (x0, y0), (x1, y1) = self.transform.transformPoints(line)
            x0 = round(x0)
            y0 = round(y0)
            x1 = round(x1)
            y1 = round(y1)
            if guideline.selected:
                gc.SetPen(self.selected_pen)
                gc.StrokeLine(x0, y0, x1, y1)
            gc.SetPen(guideline.wxPen)
            gc.StrokeLine(x0, y0, x1, y1)
            x, y = self.transform.transformPoint((guideline_x, guideline_y))
            x = round(x)
            y = round(y)
            gc.DrawEllipse(x - 3, y - 3, 6, 6)
            gc.SetBrush(self.LabelBrush)
            gc.SetPen(self.LabelPen)
            text = ""
            if guideline.name:
                text += guideline.name
            if guideline_angle in (0.0, 180.0, -180):
                if text:
                    text += f": {guideline_y:0.0f}"
                else:
                    text = f"{guideline_y:0.0f}"
            elif guideline_angle in (90.0, 270.0, -90.0):
                if text:
                    text += f": {guideline_x:0.0f}"
                else:
                    text = f"{guideline_x:0.0f}"
            else:
                if text:
                    text += f": ({guideline_x:0.0f}, {guideline_y:0.0f}) {guideline_angle:0.0f}°"
                else:
                    text = f"({guideline_x:0.0f}, {guideline_y:0.0f}) {guideline_angle:0.0f}°"
            w, h = gc.GetTextExtent(text)
            w = round(w) + 2 * s
            h = round(h) + 2 * s
            x, y = self.transform.transformPoint((guideline_x, guideline_y))
            gc.DrawRoundedRectangle(x - w - d, y + d, w, h, d)
            gc.DrawText(text, x - w, y + d + s - 1)

    def hitTestGuidelines(self, x, y, guidelines):
        delta = self.canvas.screenToCanvasXrel(4)
        for guideline in guidelines:
            distance = dist((x, y), (guideline.x, guideline.y))
            if distance <= delta:
                return guideline
            if guideline.angle in (0, 180) or (
                guideline.angle is None and guideline.x in (0, None)
            ):
                if abs(y - guideline.y) <= delta:
                    return guideline
                continue
            if guideline.angle in (90, 270) or (
                guideline.angle is None and guideline.y in (0, None)
            ):
                if abs(x - guideline.x) <= delta:
                    return guideline
                continue
            for a in (guideline.angle, guideline.angle + 180):
                angle_rad = radians(a)
                p_guide = guideline.x + distance * cos(
                    angle_rad
                ), guideline.y + distance * sin(angle_rad)
                if dist(p_guide, (x, y)) <= delta:
                    return guideline



"""
metric
===============================================================================

Glyph Commands related to glyph metric
"""
from .base import GlyphCommand
from .parameter import (
    ParamBoolRequired,
    ParamEnumeration,
    ParamIntRequired,
)


class SetWidthCommand(GlyphCommand):
    name = "Set width"
    ALLIGN_CENTER = 0
    ALLIGN_LEFT = 1
    ALLIGN_RIGHT = 2
    ALLIGN_NONE = 3
    parameters = [
        ParamIntRequired("w", "Set fixed width", 500),
        ParamEnumeration(
            "m",
            "Set margins",
            [
                "Center glyph",
                "Allign to left margin",
                "Allign to right margin",
                "Keep as is",
            ],
            ALLIGN_CENTER,
        ),
        ParamBoolRequired("c", "Preserve Composites", True),
    ]

    def _execute(self, glyph):
        glyph.width = self.w
        if self.m == self.ALLIGN_NONE:
            return
        bounds = glyph.bounds
        if bounds:
            x0, y0, x1, y1 = bounds
            if self.m == self.ALLIGN_LEFT:
                distance = (-x0, 0)
            elif self.m == self.ALLIGN_RIGHT:
                distance = (self.w - x1, 0)
            elif self.m == self.ALLIGN_CENTER:
                distance = ((self.w - x0 - x1) / 2, 0)
            if distance != (0, 0):
                glyph.moveBy(distance)
                if self.c:
                    font = glyph.font
                    if font and hasattr(font, "componentReferences"):
                        componentReferences = font.componentReferences
                        if glyph.name in componentReferences:
                            distBack = (-distance[0], 0)
                            for glyphName in componentReferences[glyph.name]:
                                compositeGlyph = font[glyphName]
                                for component in compositeGlyph.components:
                                    if component.baseGlyph == glyph.name:
                                        component.moveBy(distBack)


class CenterGlyphCommand(GlyphCommand):
    name = "Center glyph"
    parameters = [ParamBoolRequired("c", "Preserve Composites", True)]

    def _execute(self, glyph):
        bounds = glyph.bounds
        if bounds:
            x0, y0, x1, y1 = bounds
            distance = ((glyph.width - x0 - x1) / 2, 0)
            if distance != (0, 0):
                glyph.moveBy(distance)
                if self.c:
                    font = glyph.font
                    if font and hasattr(font, "componentReferences"):
                        componentReferences = font.componentReferences
                        if glyph.name in componentReferences:
                            distBack = (-distance[0], 0)
                            for glyphName in componentReferences[glyph.name]:
                                compositeGlyph = font[glyphName]
                                for component in compositeGlyph.components:
                                    if component.baseGlyph == glyph.name:
                                        component.moveBy(distBack)


class SetLeftMarginCommand(GlyphCommand):
    name = "Set left margin"
    ACTION_SET_EQUAL = 0
    ACTION_INCREASE = 1
    ACTION_DECREASE = 2
    PRESERVE_RIGHT_MARGIN = 0
    PRESERVE_WIDTH = 1
    parameters = [
        ParamEnumeration(
            "a",
            "Action",
            ["Set equal to", "Increase by", "Decrease by"],
            ACTION_SET_EQUAL,
        ),
        ParamIntRequired("v", "Units", 0),
        ParamEnumeration(
            "p", "Preserve metric", ["Right margin", "Width"], PRESERVE_RIGHT_MARGIN
        ),
        ParamBoolRequired("c", "Preserve Composites", True),
    ]

    def _execute(self, glyph):
        bounds = glyph.bounds
        if bounds:
            x0, y0, x1, y1 = bounds
            if self.p == self.PRESERVE_RIGHT_MARGIN:
                right_margin = glyph.width - x1
            if self.a == self.ACTION_SET_EQUAL:
                distance = (self.v - x0, 0)
            elif self.a == self.ACTION_INCREASE:
                distance = (self.v, 0)
            elif self.a == self.ACTION_DECREASE:
                distance = (-self.v, 0)
            if distance != (0, 0):
                glyph.moveBy(distance)
                if self.c:
                    font = glyph.font
                    if font and hasattr(font, "componentReferences"):
                        componentReferences = font.componentReferences
                        if glyph.name in componentReferences:
                            distBack = (-distance[0], 0)
                            for glyphName in componentReferences[glyph.name]:
                                compositeGlyph = font[glyphName]
                                for component in compositeGlyph.components:
                                    if component.baseGlyph == glyph.name:
                                        component.moveBy(distBack)
                if self.p == self.PRESERVE_RIGHT_MARGIN:
                    bounds_new = glyph.bounds
                    if bounds_new:
                        x0, y0, x1, y1 = bounds_new
                        glyph.width = x1 + right_margin


class SetRightMarginCommand(GlyphCommand):
    name = "Set right margin"
    ACTION_SET_EQUAL = 0
    ACTION_INCREASE = 1
    ACTION_DECREASE = 2
    PRESERVE_LEFT_MARGIN = 0
    PRESERVE_WIDTH = 1
    parameters = [
        ParamEnumeration(
            "a",
            "Action",
            ["Set equal to", "Increase by", "Decrease by"],
            ACTION_SET_EQUAL,
        ),
        ParamIntRequired("v", "Units", 0),
        ParamEnumeration(
            "p", "Preserve metric", ["Left margin", "Width"], PRESERVE_LEFT_MARGIN
        ),
        ParamBoolRequired("c", "Preserve Composites", True),
    ]

    def _execute(self, glyph):
        bounds = glyph.bounds
        if bounds:
            x0, y0, x1, y1 = bounds
            rignt_margin_old = glyph.width - x1
            if self.a == self.ACTION_SET_EQUAL:
                right_margin = self.v
            elif self.a == self.ACTION_INCREASE:
                right_margin = rignt_margin_old + self.v
            elif self.a == self.ACTION_DECREASE:
                right_margin = rignt_margin_old - self.v
            if self.p == self.PRESERVE_WIDTH:
                distance = (rignt_margin_old - right_margin, 0)
                if distance != (0, 0):
                    glyph.moveBy(distance)
                    if self.c:
                        font = glyph.font
                        if font and hasattr(font, "componentReferences"):
                            componentReferences = font.componentReferences
                            if glyph.name in componentReferences:
                                distBack = (-distance[0], 0)
                                for glyphName in componentReferences[glyph.name]:
                                    compositeGlyph = font[glyphName]
                                    for component in compositeGlyph.components:
                                        if component.baseGlyph == glyph.name:
                                            component.moveBy(distBack)
            elif self.p == self.PRESERVE_LEFT_MARGIN:
                glyph.width = x1 + right_margin



# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.0-4761b0c)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

from ..control import FontSelectListCtrl
import wx
import wx.xrc

###########################################################################
## Class SelectFontsDialogUI
###########################################################################

class SelectFontsDialogUI ( wx.Dialog ):

	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Select Fonts", pos = wx.DefaultPosition, size = wx.Size( 700,300 ), style = wx.CAPTION|wx.RESIZE_BORDER )

		self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )

		sizer = wx.BoxSizer( wx.VERTICAL )

		self.label_message = wx.StaticText( self, wx.ID_ANY, u"Select Fonts", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_LEFT|wx.ST_ELLIPSIZE_MIDDLE )
		self.label_message.Wrap( -1 )

		sizer.Add( self.label_message, 0, wx.ALL|wx.EXPAND, 10 )

		self.listCtrl_fonts = FontSelectListCtrl( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LC_REPORT|wx.LC_VIRTUAL )
		sizer.Add( self.listCtrl_fonts, 1, wx.EXPAND, 0 )

		sdbSizer = wx.StdDialogButtonSizer()
		self.sdbSizerOK = wx.Button( self, wx.ID_OK )
		sdbSizer.AddButton( self.sdbSizerOK )
		self.sdbSizerCancel = wx.Button( self, wx.ID_CANCEL )
		sdbSizer.AddButton( self.sdbSizerCancel )
		sdbSizer.Realize();

		sizer.Add( sdbSizer, 0, wx.ALL|wx.EXPAND, 5 )


		self.SetSizer( sizer )
		self.Layout()

		self.Centre( wx.BOTH )

		# Connect Events
		self.Bind( wx.EVT_KEY_DOWN, self.on_KEY_DOWN )
		self.listCtrl_fonts.Bind( wx.EVT_KEY_DOWN, self.on_KEY_DOWN )

	def __del__( self ):
		pass


	# Virtual event handlers, override them in your derived class
	def on_KEY_DOWN( self, event ):
		event.Skip()






# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.9.0 Oct 29 2020)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class RevertFontDialogUI
###########################################################################

class RevertFontDialogUI ( wx.Dialog ):

    def __init__( self, parent ):
        wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Revert Font", pos = wx.DefaultPosition, size = wx.Size( 350,400 ), style = wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER )

        self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )

        sizer_main = wx.BoxSizer( wx.VERTICAL )

        self.lbl_message = wx.StaticText( self, wx.ID_ANY, u"Select Font Components to Revert", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_message.Wrap( -1 )

        sizer_main.Add( self.lbl_message, 0, wx.ALL, 5 )

        self.checkBox_all = wx.CheckBox( self, wx.ID_ANY, u"Revert Everything", wx.DefaultPosition, wx.DefaultSize, 0 )
        sizer_main.Add( self.checkBox_all, 0, wx.ALL, 5 )

        self.m_staticline1 = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
        sizer_main.Add( self.m_staticline1, 0, wx.EXPAND, 5 )

        sizer_ctrl = wx.BoxSizer( wx.HORIZONTAL )

        sizer_check = wx.BoxSizer( wx.VERTICAL )

        self.checkBox_font_lib = wx.CheckBox( self, wx.ID_ANY, u"Font Lib", wx.DefaultPosition, wx.DefaultSize, 0 )
        sizer_check.Add( self.checkBox_font_lib, 0, wx.ALL|wx.EXPAND, 5 )

        self.checkBox_font_info = wx.CheckBox( self, wx.ID_ANY, u"Font Info", wx.DefaultPosition, wx.DefaultSize, 0 )
        sizer_check.Add( self.checkBox_font_info, 0, wx.ALL|wx.EXPAND, 5 )

        self.checkBox_groups = wx.CheckBox( self, wx.ID_ANY, u"Groups", wx.DefaultPosition, wx.DefaultSize, 0 )
        sizer_check.Add( self.checkBox_groups, 0, wx.ALL, 5 )

        self.checkBox_kerning = wx.CheckBox( self, wx.ID_ANY, u"Kerning", wx.DefaultPosition, wx.DefaultSize, 0 )
        sizer_check.Add( self.checkBox_kerning, 0, wx.ALL|wx.EXPAND, 5 )

        self.checkBox_features = wx.CheckBox( self, wx.ID_ANY, u"Features", wx.DefaultPosition, wx.DefaultSize, 0 )
        sizer_check.Add( self.checkBox_features, 0, wx.ALL|wx.EXPAND, 5 )

        self.checkBox_images = wx.CheckBox( self, wx.ID_ANY, u"Images", wx.DefaultPosition, wx.DefaultSize, 0 )
        sizer_check.Add( self.checkBox_images, 0, wx.ALL|wx.EXPAND, 5 )

        self.checkBox_data = wx.CheckBox( self, wx.ID_ANY, u"Data", wx.DefaultPosition, wx.DefaultSize, 0 )
        sizer_check.Add( self.checkBox_data, 0, wx.ALL|wx.EXPAND, 5 )


        sizer_ctrl.Add( sizer_check, 0, wx.EXPAND, 5 )

        sizer_layer = wx.BoxSizer( wx.VERTICAL )

        self.lbl_layer = wx.StaticText( self, wx.ID_ANY, u"Layers", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_layer.Wrap( -1 )

        sizer_layer.Add( self.lbl_layer, 0, wx.ALL|wx.EXPAND, 5 )

        self.checkBox_defaultLayer = wx.CheckBox( self, wx.ID_ANY, u"Default Layer", wx.DefaultPosition, wx.DefaultSize, 0 )
        sizer_layer.Add( self.checkBox_defaultLayer, 0, wx.ALL|wx.EXPAND, 5 )

        self.checkBox_layer_order = wx.CheckBox( self, wx.ID_ANY, u"Layer Order", wx.DefaultPosition, wx.DefaultSize, 0 )
        sizer_layer.Add( self.checkBox_layer_order, 0, wx.ALL|wx.EXPAND, 5 )

        self.scrolledWindow_layers = wx.ScrolledWindow( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.ALWAYS_SHOW_SB|wx.VSCROLL )
        self.scrolledWindow_layers.SetScrollRate( 5, 5 )
        sizer_layer.Add( self.scrolledWindow_layers, 1, wx.EXPAND |wx.ALL, 5 )


        sizer_ctrl.Add( sizer_layer, 1, wx.EXPAND, 5 )


        sizer_main.Add( sizer_ctrl, 1, wx.EXPAND, 5 )

        sizer_buttons = wx.StdDialogButtonSizer()
        self.sizer_buttonsOK = wx.Button( self, wx.ID_OK )
        sizer_buttons.AddButton( self.sizer_buttonsOK )
        self.sizer_buttonsCancel = wx.Button( self, wx.ID_CANCEL )
        sizer_buttons.AddButton( self.sizer_buttonsCancel )
        sizer_buttons.Realize();

        sizer_main.Add( sizer_buttons, 0, wx.ALL|wx.EXPAND, 5 )


        self.SetSizer( sizer_main )
        self.Layout()

        self.Centre( wx.BOTH )

        # Connect Events
        self.checkBox_all.Bind( wx.EVT_CHECKBOX, self.on_checkBox_all )
        self.checkBox_font_lib.Bind( wx.EVT_CHECKBOX, self.on_checkBox_single )
        self.checkBox_font_info.Bind( wx.EVT_CHECKBOX, self.on_checkBox_single )
        self.checkBox_groups.Bind( wx.EVT_CHECKBOX, self.on_checkBox_single )
        self.checkBox_kerning.Bind( wx.EVT_CHECKBOX, self.on_checkBox_single )
        self.checkBox_features.Bind( wx.EVT_CHECKBOX, self.on_checkBox_single )
        self.checkBox_images.Bind( wx.EVT_CHECKBOX, self.on_checkBox_single )
        self.checkBox_data.Bind( wx.EVT_CHECKBOX, self.on_checkBox_single )
        self.checkBox_defaultLayer.Bind( wx.EVT_CHECKBOX, self.on_checkBox_single )
        self.checkBox_layer_order.Bind( wx.EVT_CHECKBOX, self.on_checkBox_single )

    def __del__( self ):
        pass


    # Virtual event handlers, overide them in your derived class
    def on_checkBox_all( self, event ):
        event.Skip()

    def on_checkBox_single( self, event ):
        event.Skip()










###########################################################################
## Class LayerPanelUI
###########################################################################

class LayerPanelUI ( wx.Panel ):

    def __init__( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.TAB_TRAVERSAL, name = wx.EmptyString ):
        wx.Panel.__init__ ( self, parent, id = id, pos = pos, size = size, style = style, name = name )

        sizer_main = wx.BoxSizer( wx.VERTICAL )

        self.lbl_name = wx.StaticText( self, wx.ID_ANY, u"Layer:", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_name.Wrap( -1 )

        sizer_main.Add( self.lbl_name, 0, wx.ALL|wx.EXPAND, 5 )

        self.checkBox_info = wx.CheckBox( self, wx.ID_ANY, u"Info", wx.DefaultPosition, wx.DefaultSize, 0 )
        sizer_main.Add( self.checkBox_info, 0, wx.ALL|wx.EXPAND, 5 )

        self.lbl_glyphs = wx.StaticText( self, wx.ID_ANY, u"Glyphs", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_glyphs.Wrap( -1 )

        sizer_main.Add( self.lbl_glyphs, 0, wx.ALL|wx.EXPAND, 5 )

        checkList_glyphNamesChoices = []
        self.checkList_glyphNames = wx.CheckListBox( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, checkList_glyphNamesChoices, wx.LB_ALWAYS_SB|wx.LB_MULTIPLE )
        sizer_main.Add( self.checkList_glyphNames, 1, wx.ALL|wx.EXPAND, 5 )

        self.staticline = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
        sizer_main.Add( self.staticline, 0, wx.BOTTOM|wx.EXPAND|wx.TOP, 5 )


        self.SetSizer( sizer_main )
        self.Layout()
        sizer_main.Fit( self )

    def __del__( self ):
        pass





"""
config
===============================================================================
"""
import wx

pluginName = "ufo"

NODESHAPE_CIRCLE = 0
NODESHAPE_SQUARE = 1
NODESHAPE_DIAMOND = 2
NODESHAPE_TRIANGLE = 3
NODESHAPE_STAR = 4

# read values pens and brushes from the config


def cfgBrush(name, default):
    cfg = wx.GetApp().config
    cfg.SetPath("/Plugin/%s/GlyphNodes/%s" % (pluginName, name))
    if cfg.HasEntry("brush"):
        try:
            r, g, b, a, style = [int(c) for c in cfg.Read("brush").split()]
            return wx.TheBrushList.FindOrCreateBrush(wx.Colour(r, g, b, a), style)
        except:
            return default
    return default


def cfgPen(name, default):
    cfg = wx.GetApp().config
    cfg.SetPath("/Plugin/%s/GlyphNodes/%s" % (pluginName, name))
    if cfg.HasEntry("pen"):
        try:
            r, g, b, a, width, style = [int(c) for c in cfg.Read("pen").split()]
            return wx.ThePenList.FindOrCreatePen(wx.Colour(r, g, b, a), width, style)
        except:
            return default
    return default


def cfgShape(name, default):
    cfg = wx.GetApp().config
    cfg.SetPath("/Plugin/%s/GlyphNodes/%s" % (pluginName, name))
    if cfg.HasEntry("shape"):
        try:
            return cfg.ReadInt("shape")
        except:
            return default
    return default


def cfgSize(name, default):
    cfg = wx.GetApp().config
    cfg.SetPath("/Plugin/%s/GlyphNodes/%s" % (pluginName, name))
    if cfg.HasEntry("size"):
        try:
            return cfg.ReadInt("size")
        except:
            return default
    return default


def cfgRedArrowOption():
    result = {}
    cfg = wx.GetApp().config
    cfg.SetPath("/Plugin/%s/RedArrow" % (pluginName))
    result["collinear_vectors_max_distance"] = cfg.ReadInt(
        "collinear_vectors_max_distance", 2
    )
    result["extremum_calculate_badness"] = cfg.ReadBool(
        "extremum_calculate_badness", True
    )
    result["extremum_ignore_badness_below"] = cfg.ReadInt(
        "extremum_ignore_badness_below", 1
    )
    result["fractional_ignore_point_zero"] = cfg.ReadBool(
        "fractional_ignore_point_zero", True
    )
    result["smooth_connection_max_distance"] = cfg.ReadInt(
        "smooth_connection_max_distance", 4
    )
    result["semi_hv_vectors_min_distance"] = cfg.ReadInt(
        "semi_hv_vectors_min_distance", 30
    )
    result["zero_handles_max_distance"] = cfg.ReadInt("zero_handles_max_distance", 0)
    return result


def cfgRedArrowTest():
    all_tests = [
        "test_extrema",
        "test_fractional_coords",
        "test_fractional_transform",
        "test_smooth",
        "test_empty_segments",
        "test_collinear",
        "test_semi_hv",
        "test_closepath",
        "test_zero_handles",
        "test_bbox_handles",
    ]
    result = []
    cfg = wx.GetApp().config
    cfg.SetPath("/Plugin/%s/RedArrow" % (pluginName))
    for name in all_tests:
        if cfg.ReadBool(name, True):
            result.append(name)
    return result




import logging
import wx
from .fontInfoLegalUI import FontInfoLegalUI
log = logging.getLogger(__name__)

class FontInfoLegal(FontInfoLegalUI):
    def __init__(
        self,
        parent,
        id=wx.ID_ANY,
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=wx.BORDER_NONE | wx.TAB_TRAVERSAL,
    ):
        super().__init__(parent, id, pos, size, style, name="FontInfoLegal")
        self.SetScrollRate(5, 5)

    def __repr__(self):
        return f"<{self.__class__.__name__} of {self.Parent}>"



# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.0-4761b0c)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class AnchorDialogUI
###########################################################################

class AnchorDialogUI ( wx.Dialog ):

	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Anchor", pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.CAPTION|wx.RESIZE_BORDER )

		self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )

		sizerMain = wx.GridBagSizer( 0, 0 )
		sizerMain.SetFlexibleDirection( wx.VERTICAL )
		sizerMain.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

		self.message = wx.StaticText( self, wx.ID_ANY, u"Edit Anchor", wx.DefaultPosition, wx.DefaultSize, wx.ST_NO_AUTORESIZE|wx.ALIGN_CENTER_HORIZONTAL )
		self.message.Wrap( -1 )

		sizerMain.Add( self.message, wx.GBPosition( 0, 0 ), wx.GBSpan( 1, 2 ), wx.ALL|wx.EXPAND, 5 )

		self.label_name = wx.StaticText( self, wx.ID_ANY, u"Name", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.label_name.Wrap( -1 )

		sizerMain.Add( self.label_name, wx.GBPosition( 1, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.textCtrl_name = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		sizerMain.Add( self.textCtrl_name, wx.GBPosition( 1, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.label_position = wx.StaticText( self, wx.ID_ANY, u"Position", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.label_position.Wrap( -1 )

		sizerMain.Add( self.label_position, wx.GBPosition( 2, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.EXPAND, 5 )

		sizerPos = wx.FlexGridSizer( 0, 2, 0, 0 )
		sizerPos.SetFlexibleDirection( wx.BOTH )
		sizerPos.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

		self.label_x = wx.StaticText( self, wx.ID_ANY, u"x", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.label_x.Wrap( -1 )

		sizerPos.Add( self.label_x, 0, wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.spinCtrl_x = wx.SpinCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_RIGHT|wx.SP_ARROW_KEYS, -10000, 10000, 0 )
		sizerPos.Add( self.spinCtrl_x, 0, wx.ALL, 5 )

		self.label_y = wx.StaticText( self, wx.ID_ANY, u"y", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.label_y.Wrap( -1 )

		sizerPos.Add( self.label_y, 0, wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.spinCtrl_y = wx.SpinCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_RIGHT|wx.SP_ARROW_KEYS, -10000, 10000, 0 )
		sizerPos.Add( self.spinCtrl_y, 0, wx.ALL, 5 )


		sizerMain.Add( sizerPos, wx.GBPosition( 2, 1 ), wx.GBSpan( 1, 1 ), wx.EXPAND, 5 )

		self.label_colour = wx.StaticText( self, wx.ID_ANY, u"Colour", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.label_colour.Wrap( -1 )

		sizerMain.Add( self.label_colour, wx.GBPosition( 3, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 5 )

		sizerColour = wx.BoxSizer( wx.HORIZONTAL )

		self.checkBox_color = wx.CheckBox( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		sizerColour.Add( self.checkBox_color, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.colourPicker = wx.ColourPickerCtrl( self, wx.ID_ANY, wx.Colour( 255, 0, 0 ), wx.DefaultPosition, wx.DefaultSize, wx.CLRP_DEFAULT_STYLE )
		self.colourPicker.Enable( False )

		sizerColour.Add( self.colourPicker, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


		sizerMain.Add( sizerColour, wx.GBPosition( 3, 1 ), wx.GBSpan( 1, 1 ), wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 0 )

		self.m_staticline1 = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
		sizerMain.Add( self.m_staticline1, wx.GBPosition( 4, 0 ), wx.GBSpan( 1, 2 ), wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 0 )

		dialogButtons = wx.StdDialogButtonSizer()
		self.dialogButtonsOK = wx.Button( self, wx.ID_OK )
		dialogButtons.AddButton( self.dialogButtonsOK )
		self.dialogButtonsCancel = wx.Button( self, wx.ID_CANCEL )
		dialogButtons.AddButton( self.dialogButtonsCancel )
		dialogButtons.Realize();

		sizerMain.Add( dialogButtons, wx.GBPosition( 5, 0 ), wx.GBSpan( 1, 2 ), wx.ALL|wx.EXPAND, 5 )


		sizerMain.AddGrowableCol( 1 )

		self.SetSizer( sizerMain )
		self.Layout()
		sizerMain.Fit( self )

		self.Centre( wx.BOTH )

		# Connect Events
		self.colourPicker.Bind( wx.EVT_UPDATE_UI, self.on_update_colourPicker )

	def __del__( self ):
		pass


	# Virtual event handlers, override them in your derived class
	def on_update_colourPicker( self, event ):
		event.Skip()





# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.0-4761b0c)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class FontInfoNameUI
###########################################################################

class FontInfoNameUI ( wx.ScrolledWindow ):

    def __init__( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 437,924 ), style = wx.HSCROLL|wx.TAB_TRAVERSAL|wx.VSCROLL, name = wx.EmptyString ):
        wx.ScrolledWindow.__init__ ( self, parent, id = id, pos = pos, size = size, style = style, name = name )

        self.SetExtraStyle( wx.WS_EX_VALIDATE_RECURSIVELY )
        self.SetBackgroundColour( wx.Colour( 255, 255, 255 ) )

        sizer = wx.BoxSizer( wx.VERTICAL )

        boxMain = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Main" ), wx.VERTICAL )

        gridMain = wx.FlexGridSizer( 0, 2, 0, 0 )
        gridMain.AddGrowableCol( 1 )
        gridMain.SetFlexibleDirection( wx.HORIZONTAL )
        gridMain.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_familyName = wx.StaticText( self, wx.ID_ANY, u"Family Name", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_familyName.Wrap( -1 )

        self.lbl_familyName.SetMinSize( wx.Size( 170,-1 ) )

        gridMain.Add( self.lbl_familyName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_familyName = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"familyName" )
        gridMain.Add( self.txtCtrl_familyName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )

        self.lbl_styleName = wx.StaticText( self, wx.ID_ANY, u"Style Name", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_styleName.Wrap( -1 )

        gridMain.Add( self.lbl_styleName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_styleName = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"styleName" )
        gridMain.Add( self.txtCtrl_styleName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )


        boxMain.Add( gridMain, 0, wx.EXPAND, 5 )


        sizer.Add( boxMain, 0, wx.ALL|wx.EXPAND, 5 )

        boxStylemap = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"StyleMap" ), wx.VERTICAL )

        gridStylemap = wx.FlexGridSizer( 0, 2, 0, 0 )
        gridStylemap.AddGrowableCol( 1 )
        gridStylemap.SetFlexibleDirection( wx.HORIZONTAL )
        gridStylemap.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_styleMapFamilyName = wx.StaticText( self, wx.ID_ANY, u"Family Name", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_styleMapFamilyName.Wrap( -1 )

        self.lbl_styleMapFamilyName.SetMinSize( wx.Size( 170,-1 ) )

        gridStylemap.Add( self.lbl_styleMapFamilyName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_styleMapFamilyName = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"styleMapFamilyName" )
        gridStylemap.Add( self.txtCtrl_styleMapFamilyName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )

        self.lbl_styleMapStyleName = wx.StaticText( self, wx.ID_ANY, u"Style Name", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_styleMapStyleName.Wrap( -1 )

        gridStylemap.Add( self.lbl_styleMapStyleName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        choice_styleMapStyleNameChoices = [ wx.EmptyString, u"regular", u"italic", u"bold", u"bold italic" ]
        self.choice_styleMapStyleName = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, choice_styleMapStyleNameChoices, 0, wx.DefaultValidator, u"styleMapStyleName" )
        self.choice_styleMapStyleName.SetSelection( 0 )
        gridStylemap.Add( self.choice_styleMapStyleName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


        boxStylemap.Add( gridStylemap, 0, wx.EXPAND, 5 )


        sizer.Add( boxStylemap, 0, wx.ALL|wx.EXPAND, 5 )

        boxOpenType = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"OpenType" ), wx.VERTICAL )

        gridOpenType = wx.FlexGridSizer( 0, 2, 0, 0 )
        gridOpenType.AddGrowableCol( 1 )
        gridOpenType.SetFlexibleDirection( wx.HORIZONTAL )
        gridOpenType.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_openTypeNamePreferredFamilyName = wx.StaticText( self, wx.ID_ANY, u"Typographic Family Name", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeNamePreferredFamilyName.Wrap( -1 )

        self.lbl_openTypeNamePreferredFamilyName.SetMinSize( wx.Size( 170,-1 ) )

        gridOpenType.Add( self.lbl_openTypeNamePreferredFamilyName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeNamePreferredFamilyName = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeNamePreferredFamilyName" )
        gridOpenType.Add( self.txtCtrl_openTypeNamePreferredFamilyName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )

        self.lbl_openTypeNamePreferredSubfamilyName = wx.StaticText( self, wx.ID_ANY, u"Typographic Subfamily Name", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeNamePreferredSubfamilyName.Wrap( -1 )

        gridOpenType.Add( self.lbl_openTypeNamePreferredSubfamilyName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeNamePreferredSubfamilyName = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeNamePreferredSubfamilyName" )
        gridOpenType.Add( self.txtCtrl_openTypeNamePreferredSubfamilyName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )

        self.lbl_openTypeNameCompatibleFullName = wx.StaticText( self, wx.ID_ANY, u"Compatible Full Name", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeNameCompatibleFullName.Wrap( -1 )

        gridOpenType.Add( self.lbl_openTypeNameCompatibleFullName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeNameCompatibleFullName = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeNameCompatibleFullName" )
        gridOpenType.Add( self.txtCtrl_openTypeNameCompatibleFullName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )

        self.lbl_openTypeOS2Selection_8 = wx.StaticText( self, wx.ID_ANY, u"OS/2 Selection Bit 8", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeOS2Selection_8.Wrap( -1 )

        gridOpenType.Add( self.lbl_openTypeOS2Selection_8, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.checkBox_openTypeOS2Selection_8 = wx.CheckBox( self, wx.ID_ANY, u"Names are WWS compatible", wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2Selection 8" )
        gridOpenType.Add( self.checkBox_openTypeOS2Selection_8, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )

        self.lbl_openTypeNameWWSFamilyName = wx.StaticText( self, wx.ID_ANY, u"WWS Family Name", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeNameWWSFamilyName.Wrap( -1 )

        gridOpenType.Add( self.lbl_openTypeNameWWSFamilyName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeNameWWSFamilyName = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeNameWWSFamilyName" )
        gridOpenType.Add( self.txtCtrl_openTypeNameWWSFamilyName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )

        self.lbl_openTypeNameWWSSubfamilyName = wx.StaticText( self, wx.ID_ANY, u"WWS Subfamily Name", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeNameWWSSubfamilyName.Wrap( -1 )

        gridOpenType.Add( self.lbl_openTypeNameWWSSubfamilyName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeNameWWSSubfamilyName = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeNameWWSSubfamilyName" )
        gridOpenType.Add( self.txtCtrl_openTypeNameWWSSubfamilyName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )

        self.m_staticText16 = wx.StaticText( self, wx.ID_ANY, u"OS/2 Selection Style Bits", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.m_staticText16.Wrap( -1 )

        gridOpenType.Add( self.m_staticText16, 0, wx.LEFT|wx.TOP, 5 )

        self.checkBox_openTypeOS2Selection_1 = wx.CheckBox( self, wx.ID_ANY, u"Underscored (Bit 1)", wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2Selection 1" )
        gridOpenType.Add( self.checkBox_openTypeOS2Selection_1, 0, wx.LEFT|wx.TOP, 5 )


        gridOpenType.Add( ( 0, 0), 0, 0, 0 )

        self.checkBox_openTypeOS2Selection_2 = wx.CheckBox( self, wx.ID_ANY, u"Negative (Bit 2)", wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2Selection 2" )
        gridOpenType.Add( self.checkBox_openTypeOS2Selection_2, 0, wx.LEFT|wx.TOP, 5 )


        gridOpenType.Add( ( 0, 0), 0, 0, 0 )

        self.checkBox_openTypeOS2Selection_3 = wx.CheckBox( self, wx.ID_ANY, u"Outlined (Bit 3)", wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2Selection 3" )
        gridOpenType.Add( self.checkBox_openTypeOS2Selection_3, 0, wx.LEFT|wx.TOP, 5 )


        gridOpenType.Add( ( 0, 0), 0, 0, 0 )

        self.checkBox_openTypeOS2Selection_4 = wx.CheckBox( self, wx.ID_ANY, u"Strikeout (Bit 4)", wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2Selection 4" )
        gridOpenType.Add( self.checkBox_openTypeOS2Selection_4, 0, wx.LEFT|wx.TOP, 5 )


        gridOpenType.Add( ( 0, 0), 0, 0, 0 )

        self.checkBox_openTypeOS2Selection_9 = wx.CheckBox( self, wx.ID_ANY, u"Oblique (Bit 9)", wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2Selection 9" )
        gridOpenType.Add( self.checkBox_openTypeOS2Selection_9, 0, wx.LEFT|wx.TOP, 5 )


        boxOpenType.Add( gridOpenType, 0, wx.ALL|wx.EXPAND, 5 )


        sizer.Add( boxOpenType, 0, wx.ALL|wx.EXPAND, 5 )

        boxPostScript = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"PostScript" ), wx.VERTICAL )

        gridPostScript = wx.FlexGridSizer( 0, 2, 0, 0 )
        gridPostScript.AddGrowableCol( 1 )
        gridPostScript.SetFlexibleDirection( wx.HORIZONTAL )
        gridPostScript.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_postscriptFontName = wx.StaticText( self, wx.ID_ANY, u"Font Name", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptFontName.Wrap( -1 )

        self.lbl_postscriptFontName.SetMinSize( wx.Size( 170,-1 ) )

        gridPostScript.Add( self.lbl_postscriptFontName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_postscriptFontName = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"postscriptFontName" )
        gridPostScript.Add( self.txtCtrl_postscriptFontName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )

        self.lbl_postscriptFullName = wx.StaticText( self, wx.ID_ANY, u"Full Name", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptFullName.Wrap( -1 )

        gridPostScript.Add( self.lbl_postscriptFullName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_postscriptFullName = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"postscriptFullName" )
        gridPostScript.Add( self.txtCtrl_postscriptFullName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )

        self.lbl_postscriptWeightName = wx.StaticText( self, wx.ID_ANY, u"Weight Name", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptWeightName.Wrap( -1 )

        gridPostScript.Add( self.lbl_postscriptWeightName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.textCtrl_postscriptWeightName = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"postscriptWeightName" )
        gridPostScript.Add( self.textCtrl_postscriptWeightName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )


        boxPostScript.Add( gridPostScript, 0, wx.EXPAND, 5 )


        sizer.Add( boxPostScript, 0, wx.ALL|wx.EXPAND, 5 )

        boxMacintosh = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Macintosh" ), wx.VERTICAL )

        gridMacintosh = wx.FlexGridSizer( 0, 2, 0, 0 )
        gridMacintosh.AddGrowableCol( 1 )
        gridMacintosh.SetFlexibleDirection( wx.BOTH )
        gridMacintosh.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_macintoshFONDName = wx.StaticText( boxMacintosh.GetStaticBox(), wx.ID_ANY, u"FOND Name", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_macintoshFONDName.Wrap( -1 )

        self.lbl_macintoshFONDName.SetMinSize( wx.Size( 170,-1 ) )

        gridMacintosh.Add( self.lbl_macintoshFONDName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_macintoshFONDName = wx.TextCtrl( boxMacintosh.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"macintoshFONDName" )
        gridMacintosh.Add( self.txtCtrl_macintoshFONDName, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )


        boxMacintosh.Add( gridMacintosh, 1, wx.EXPAND, 5 )


        sizer.Add( boxMacintosh, 0, wx.ALL|wx.EXPAND, 5 )


        self.SetSizer( sizer )
        self.Layout()

    def __del__( self ):
        pass





# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version Oct 26 2018)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

from .panelItemPicker import PanelItemPicker
import wx
import wx.xrc

###########################################################################
## Class DialogItemPickerUI
###########################################################################

class DialogItemPickerUI ( wx.Dialog ):

	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"ItemPicker", pos = wx.DefaultPosition, size = wx.DefaultSize, style = wx.CAPTION|wx.CLOSE_BOX|wx.RESIZE_BORDER )

		self.SetSizeHints( wx.Size( 300,280 ), wx.DefaultSize )

		sizer_main = wx.BoxSizer( wx.VERTICAL )

		self.label_message = wx.StaticText( self, wx.ID_ANY, u"Message", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.label_message.Wrap( -1 )

		sizer_main.Add( self.label_message, 0, wx.ALL|wx.EXPAND, 5 )

		self.itemPicker = PanelItemPicker( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		sizer_main.Add( self.itemPicker, 1, wx.EXPAND, 0 )

		sdbSizer = wx.StdDialogButtonSizer()
		self.sdbSizerOK = wx.Button( self, wx.ID_OK )
		sdbSizer.AddButton( self.sdbSizerOK )
		self.sdbSizerCancel = wx.Button( self, wx.ID_CANCEL )
		sdbSizer.AddButton( self.sdbSizerCancel )
		sdbSizer.Realize()

		sizer_main.Add( sdbSizer, 0, wx.ALL|wx.EXPAND, 5 )


		self.SetSizer( sizer_main )
		self.Layout()
		sizer_main.Fit( self )

		self.Centre( wx.BOTH )

	def __del__( self ):
		pass


